unit MikApi04d;

interface
Uses Windows, SysUtils, classes, dynlibs;

// MIK-OLAP API V.4.0  Stand 02.07.99 MM GetItemsData hinzugefgt
// MIK-OLAP API V.4.0  Stand 12.05.99 MM berarbeitet

// MIK-OLAP API V.2.0  Stand 24.06.98 WGR - Versions-Synchronisation mit MIK-OLAP V.2.0
// MIK-OLAP API V.2.3  Stand 18.06.98 WGR - Funktionen OMIKLogin und OMIKGetCube gendert
// MIK-OLAP API V.2.2  Stand 04.05.98 WGR - Nachdokumentation
// MIK-OLAP API V.2.1  Stand 15.04.97 MM
// MIK-OLAP API V.1.1  Stand 27.12.97 MM
// MIK-OLAP API V.1.0  Stand 17.01.97 MM


//****************************************************************************************
//
//  C-Interface (API) for MIK-OLAP 4.0
//  ==================================
//  (C) Copyright MIK GmbH, Konstanz Germany                                              //
//
//****************************************************************************************


// Stack size should be 1 MEGABYTE !

// --------------------------------------------------------------------
//  -> = input
//  <- = ouput or result
// --------------------------------------------------------------------

var MikOlapAPI  : String = 'MIKAPIO.DLL';
    MikOlapPath : String = 'C:\MIKOLAP';

const

moApiNotFound =  -9999;
moProcNotFound = -8888;

// --------------------------------------------------------------------
// result types for all functions

moOkay               = 0; // okay
moNoMoreData         = 1; // no more data to transfer
moServerNotAvaliable = 3; // server error e.g. is not avaliable

moServerError        = -1; // severe error
moNoData             = -2; // data is not available // N/A
moServerBusy         = -3; // server is busy, no login possible at the moment
moServerBlocked1     = -4; // server is blocked, no login possible at the moment
moServerBlocked2     = -5; // server is blocked, no login possible at the moment
moServerUpdated      = -6; // server has been updated, screen must be refreshed
moUserAlreadyLogged  = -7; // user ist already logged in

// --------------------------------------------------------------------------------------
// caller type

moCALLERAPI        	  = 0;
moCALLERACTIVEMINER       = 8;
moCALLERACTIVEMINERNOMESS = 108;
moCALLERSEAGATEINFO       = 9;

// subset types
moSUBSETTYPEITEM	  = 0;

// reserved, not yet active
moSUBSETTYPEALL           = 1;
moSUBSETTYPERESERVERED	  = 2;
moSUBSETTYPEGROUP	  = 3;
moSUBSETTYPEGROUPWFATHER  = 4;
moSUBSETTYPETREE          = 5;
moSUBSETTYPETREEWFATHER   = 6;


// -------------------------------------------------------------------------------------
// cube type

moCUBETYPESTANDARD        = 0;
moCUBETYPEATTRIBUTE       = 1;
moCUBETYPECURRENCY        = 2;
moCUBETYPETEXT            = 3;
// -------------------------------------------------------------------------------------

// -------------------------------------------------------------------------------------
// dimension type

moDIMTYPEANY              = 0;
moDIMTYPEDATASET          = 1;
moDIMTYPEPERIOD           = 2;
moDIMTYPEYEAR             = 3;
moDIMTYPECURRENCY         = 4;
moDIMTYPEVARIABLE         = 5;
moDIMTYPESUBSYSTEM        = 6;
moDIMTYPEATTRIBUTE        = 7;
// -------------------------------------------------------------------------------------

// -------------------------------------------------------------------------------------
// commonly used variables
// long user   = user ID
// -------------------------------------------------------------------------------------

Type

  TLoginItem = record
     User : Longint;
     dbId : LongInt;
  end;
  TLogInList = Class(TList)
    function  GetUser(i: Integer): TLoginItem;
  public     // Public-Deklarationen
    function  AddUser(User, dbId: LongInt): Integer;
    Procedure DeleteUser(User: LongInt);
    property  Items[i: Integer] : TLoginItem read GetUser;
    property  count;
  end;

pString = ^Char;

// universal datatyp for data exchange


// typedef struct
  _params = record
             lpszTxt    : PAnsiChar;               // Pointer to null terminated string
					           // if used, caller must ensure a valid pointer
             ldummy1    : LongInt;                 // reserved
             level      : LongInt;                 // hierarchical level of the element
             dummy2     : LongInt;                 // reserved
             dummy3     : LongInt;                 // reserved
             elementid  : LongInt;                 // reserved
             dValue     : Double;                  // result value
             cUsage     : Char;                    // type of usage
             filler     : LongInt;                 // reserved
             dummy5     : LongInt;                 // reserved
           end;
  params = _params;


// subset for GetDataItems
  _dataitem = record
               item       : PChar;             // Pointer to null terminated string
				               // if used, caller must ensure a valid pointer
               subsettype : LongInt;           // reserved
              end;
  TdataItem = _dataitem;
  Tdataitems  = Array[0..18] of Tdataitem;
  pdataitems = ^Tdataitems;


pparams = ^params;                             // pointer to params
ppparams = ^pparams;                           // pointer to params
plong   = ^LongInt;                            // pointer to long data type



// ---------------------------------------------------------------------
// GetDatabase: Get list of available databases
// ->   UserNr must be 0
// ->	first = 1, ask for first database
// ->	first = 0, ask for next database
// <-   databaseid returns databaseid
// <-   databasename returns database name, must be a valid pointer
// function GetDatabase (UserNr: LongInt;databasid : plong;
//                       databasename : PChar;
//                       FIRST: LongInt): LongInt; stdcall;

TGetDatabase            = function (UserNr: LongInt;databasid : plong;databasename : PChar;
                                    FIRST: LongInt): LongInt; stdcall;
// --------------------------------------------------------------------

// --------------------------------------------------------------------------------------
// OMIKGetModel
//
// Infos ueber die Anwendunsgmodelle erfragen
// First = 1, erstes Modell erfragen
// First = 0, naechstes Modell erfragen
//
// ModID gibt Modell-ID zurueck
// ModText gibt Bezeichnung des Modells zurueck
// Ergebnis :  1 = Kein weiteres Modell vorhanden
//             0 = Weiteres Modell vorhanden
//           < 0 = Fehler
//
//long WINAPI OMIKGetModel (long  ActUser,           Benutzer-Nr.
//                          long  DatabaseID,        Datenbank-ID - wird ggw. ignoriert
//                          plong ModId,             Zeiger auf Modell-ID
//                          LPSTR ModText,           Zeiger auf Modell-Name
//                          long  FIRST);            s.o.
TGetModel               = function (UserNr: LongInt;databasid : plong;modid: plong;
                                    modtext: PChar; FIRST: LongInt): LongInt; stdcall;
// --------------------------------------------------------------------------------------


// -------------------------------------------------------------------------
// GetRequiredLoginParams: nr of necessary login params
// -> databaseid  : ID of database
// <- result        1 = only password
//                  2 = name and password
// function GetRequiredLoginParams (Databaseid : LongInt): LongInt; stdcall;

TGetRequiredLoginParams = function (Databaseid : LongInt): LongInt; stdcall;
// ---------------------------------------------------------------------


// ---------------------------------------------------------------------
// Login	: Login to the database
// -> caller      : ID of caller
// -> loginmode   : 0 = normal, 88 = second login allowed
// -> databaseid  : ID of database
// -> database    : alternativ to ID test e.g. DEMO
// -> Name        : name of user
// -> Password    : password of user
// -> Language    : Language of user
//		    0 =	base language
//		    1 .. 10, one of the 10 switchable languages
//		    only for MIK-OLAP
// <- result        usernr (userhandle) >0, on error zero
// function Login (Caller: LongInt; loginmode: LongInt;
//                 DataBaseID: LongInt; Database: PChar; Name: PChar;
//                 Password: PChar; Language: LongInt): LongInt; stdcall;

TLogin =                  function (Caller: LongInt; loginmode: LongInt; DataBaseID: LongInt;
                                    Database: PChar; Name: PChar; Password: PChar; Language: LongInt): LongInt; stdcall;
// ---------------------------------------------------------------------


// --------------------------------------------------------------------------
// Logout	  : Logout from the database
// -> Actuser     : ID of user
// -> databaseid  : ID of database
// function Logout (ActUser: LongInt; DatabaseID: LongInt): LongInt; stdcall;
TLogout =                 function (ActUser: LongInt; DatabaseID: LongInt): LongInt; stdcall;
// ---------------------------------------------------------------------


// ---------------------------------------------------------------------------------
// GetCube	  : Get infos about availabale cubes
// -> Actuser     : ID of user
// -> databaseid  : ID of database
// -> FIRST       : 1 if first query, 0 if further query
// <- CubeID      : ID of the cube
// <- CubeType    : type of the cube, see above
// <- CubeText    : name of the cube, must be a valid pointer
// GetCube (ActUser: LongInt; DatabaseID: LongInt; CubeId: pLong;
//           CubeType: pLong; CubeText: pString; FIRST: LongInt): LongInt; stdcall;

TGetCube =                function (ActUser: LongInt; DatabaseID: LongInt; CubeId: pLong;
                                    CubeType: pLong; CubeText: pString; FIRST: LongInt): LongInt; stdcall;
// ---------------------------------------------------------------------



// ---------------------------------------------------------------------------------
// GetDimension   : Get infos about the dimensions of a cube
// -> Actuser     : ID of user
// -> databaseid  : ID of database
// -> CubeID      : ID of the cube to be queried
// -> FIRST       : 1 if first query, 0 if further query
// <- DimID       : ID of the dimension to be queried
// <- DimType     : type of the dimension, see above
// <- DimText     : name of the dimension, must be a valid pointer
// function GetDimension (ActUser: LongInt; DatabaseID: LongInt; CubeID: LongInt;
//                        DimId: pLong; DimType: pLong; DimText: pString;
//                        FIRST: LongInt): LongInt; stdcall;

TGetDimension =           function (ActUser: LongInt; DatabaseID: LongInt; CubeID: LongInt;
                                    DimId: pLong; DimType: pLong; DimText: pString; FIRST: LongInt): LongInt; stdcall;
// ---------------------------------------------------------------------


// -------------------------------------------------------------------------------
// SetElementInfo : Prepares the query of the elements of a dimension
// -> Actuser     : Benutzer-Nr.
// -> databaseid  : Datenbank-ID - wird ggw. ignoriert
// -> CubeID      : Wuerfel-ID
// -> DimID       : Dimensions-ID
// -> HierID      : Hierarchie-ID, reserviert
// -> Condition   : Zeiger auf Bedingungs-Struktur, must be filled with
//                  zero
// SetElementInfo (ActUser: LongInt; DatabaseID: LongInt; CubeID: LongInt;
//                 DimID: LongInt; HierID: LongInt; Condition: pParams): LongInt;
//                 stdcall;

TSetElementInfo =         function (ActUser: LongInt; DatabaseID: LongInt; CubeID: LongInt;
                                    DimID: LongInt; HierID: LongInt; Condition: pParams): LongInt; stdcall;
// ---------------------------------------------------------------------


// ---------------------------------------------------------------------
// GetElement : gets one element according to infos in SetElementInfo
// -> Actuser     : ID of user
// <- Result      : level and lpsztxt are set
//			      lpsztxt must be a valid pointer
// GetElement  (ActUser: LongInt; Info: pParams): LongInt; stdcall;

TGetElement =             function (ActUser: LongInt; Info: pParams): LongInt; stdcall;
// ---------------------------------------------------------------------


// ---------------------------------------------------------------------------------------------
// SetDataInfo    : Prepares the query of the elements of a dimension
//                  this function has to be called for every dimension
//		    of the cube
//		    if cusage =1 then lpsztxt contains the element of
//		    the specified dimension
//		    if cusage =0 then lpsztxt contains nothing and
//		    all elements of the dimension are meant (wildcard
// 		    no mare than 2 wildcards can be used
// -> Actuser     : ID of user
// -> databaseid  : ID of database
// -> CubeID      : ID of the cube to be
// -> DimID       : ID of the dimension to be queried
// -> HierID      : reserved
// -> Condition   : must be set with cUsahe & lpsztxt (valid pointer)
// function SetDataInfo (ActUser: LongInt; DatabaseID: LongInt; CubeID: LongInt;
//                       DimID: LongInt; HierID: LongInt; Condition: pParams): LongInt; stdcall;

TSetDataInfo =            function (ActUser: LongInt; DatabaseID: LongInt; CubeID: LongInt;
                                    DimID: LongInt; HierID: LongInt; Condition: pParams): LongInt; stdcall;
// ---------------------------------------------------------------------


// ----------------------------------------------------------------------
// GetData : gets data according to infos in SetDataInfo
// -> Actuser     : ID of user
// <- Result      : level and lpsztxt are set
//			      lpsztxt must be a valid pointer
// function GetData  (ActUser: LongInt; Info: pParams): LongInt; stdcall;

TGetData =                function (ActUser: LongInt; Info: pParams): LongInt; stdcall;
// ---------------------------------------------------------------------


// ---------------------------------------------------------------------------------
// this is an alternative to SetDataInfo etc. There is only one call
// necessary to get data, but there can be no wildcards set
// GetDataItems : gets data
// -> Actuser     : ID of user
// -> cubeid      : ID of cube
// -> options     : type of option
// <- Result      : dValue ist set
// function GetDataItems  (ActUser: LongInt; cubeid: LongInt; options: LongInt;
//                         dataitems: pdataitems; Info: ppParams): LongInt; stdcall;

TGetDataItems =           function (ActUser: LongInt; cubeid: LongInt; options: LongInt;
                                    dataitems: pdataitems; Info: ppParams): LongInt; stdcall;
// ---------------------------------------------------------------------


// ---------------------------------------------------------------------------------
// this is an alternative to SetDataInfo etc. There is only one call
// necessary to get data, there can be set wildcards
// GetMultipleData : gets data
// -> Actuser     : ID of user
// -> cubeid      : ID of cube
// -> options     : type of option
// -> basic       : 1 = get only basic data
// <-> dataitems  : input  = item of element & subsettype
// <-> dataitems  : output = element texts of dimensions
// <- Result      : dValue ist set
// <- FIRST       : FIRST query
// function GetMultipleData (ActUser: LongInt; cubeid: LongInt; options: LongInt;
//                            basic: LongInt; dataitems: pDataItems; info: ppParams;
//                            FIRST: LongInt): LongInt;

TGetMultipleData =        function (ActUser: LongInt; cubeid: LongInt; options: LongInt;
                                    basic: LongInt; dataitems: pDataItems; info: ppParams;
                                    FIRST: LongInt): LongInt;


// --------------------------------------------------------------------------
// SendData : puts data into the database according to infos in SetDataInfo
// -> Actuser     : ID of user
// <- Result      : level and lpsztxt are set
//			      lpsztxt must be a valid pointer
// SendData  (ActUser: LongInt; Info: pParams): LongInt; stdcall;

TSendData =               function (ActUser: LongInt; Info: pParams): LongInt; stdcall;


// ----------------------------------------------------------------------------------
// this is an alternative to PutData etc. There is only one call
// necessary to put data, but there can be no wildcards set
// PutDataItems : gets data according to infos in SetDataInfo
// -> Actuser     : ID of user
// -> cubeid      : ID of cube
// -> options     : type of option
// -> Result      : dValue has to be set
// function PutDataItems  (ActUser: LongInt; cubeid: LongInt; options: LongInt;
//                          dataitems: pDataItems; Info: ppParams): LongInt; stdcall;

TPutDataItems =           function (ActUser: LongInt; cubeid: LongInt; options: LongInt;
                                    dataitems: pDataItems; Info: ppParams): LongInt; stdcall;
// ---------------------------------------------------------------------------


// --------------------------------------------------------------------------------------
// GetAttribute: gets the attributes of a given element
// -> Actuser     : ID of user
// -> DatabaseID  : ID of the database
// -> dimid       : ID of the element dimension
// -> elementname : element string
// -> attrdimid   : ID of the attribute class
// -> resulttext  : text of the attribute
// -> FIRST       : FIRST request
// function GetAttribute (ActUser: LongInt; DatabaseID: LongInt; dimid: LongInt;
//                        elementname: pString; attrdimid: LongInt; resulttext: pString;
//                        FIRST: LongInt): LongInt; stdcall;


TGetAttribute =           function (ActUser: LongInt; DatabaseID: LongInt; dimid: LongInt;
                                    elementname: pString; attrdimid: LongInt; resulttext: pString;
                                    FIRST: LongInt): LongInt; stdcall;

TGetBasicData =           function (ActUser: LONGINT; DatabaseID: LONGINT; dataitems: pDataitems;
                                    Info: ppParams; FIRST: LONGINT ): LONGINT; stdcall;


function moGetDatabase            (UserNr: LongInt;databasid : plong; databasename : PChar; FIRST: LongInt): LongInt;
function moGetRequiredLoginParams (Databaseid : LongInt): LongInt;
function moLogin                  (Caller: LongInt; loginmode: LongInt; DataBaseID: LongInt; Database: PChar; Name: PChar; Password: PChar; Language: LongInt): LongInt;
function moLogout                 (ActUser: LongInt; DatabaseID: LongInt): LongInt;
function moGetModel               (UserNr: LongInt;databasid : plong;modid: plong; modtext: PChar; FIRST: LongInt): LongInt;
function moGetCube                (ActUser: LongInt; DatabaseID: LongInt; CubeId: pLong; CubeType: pLong; CubeText: pString; FIRST: LongInt): LongInt;
function moGetDimension           (ActUser: LongInt; DatabaseID: LongInt; CubeID: LongInt; DimId: pLong; DimType: pLong; DimText: pString; FIRST: LongInt): LongInt;
function moSetElementInfo         (ActUser: LongInt; DatabaseID: LongInt; CubeID: LongInt; DimID: LongInt; HierID: LongInt; Condition: pParams): LongInt;
function moGetElement             (ActUser: LongInt; Info: pParams): LongInt;
function moSetDataInfo            (ActUser: LongInt; DatabaseID: LongInt; CubeID: LongInt; DimID: LongInt; HierID: LongInt; Condition: pParams): LongInt;
function moGetData                (ActUser: LongInt; Info: pParams): LongInt;
function moGetDataItems           (ActUser: LongInt; cubeid: LongInt; options: LongInt; dataitems: pdataitems; Info: ppParams): LongInt;
function moGetMultipleData        (ActUser: LongInt; cubeid: LongInt; options: LongInt; basic: LongInt; dataitems: pDataItems; info: ppParams; FIRST: LongInt): LongInt;
function moSendData               (ActUser: LongInt; Info: pParams): LongInt;
function moPutDataItems           (ActUser: LongInt; cubeid: LongInt; options: LongInt; dataitems: pDataItems; Info: ppParams): LongInt;
function moGetAttribute           (ActUser: LongInt; DatabaseID: LongInt; dimid: LongInt; elementname: pString; attrdimid: LongInt; resulttext: pString; FIRST: LongInt): LongInt;
function moGetBasicData           (ActUser: LONGINT; DatabaseID: LONGINT; dataitems: pDataitems; Info: ppParams; FIRST: LONGINT ): LONGINT;


implementation

var hndAPI   : THandle;
    ExitSave : Pointer;
    GetDatabase : TGetDataBase;
    GetRequiredLoginParams : TGetRequiredLoginParams;
    Login : TLogin;
    Logout : TLogout;
    GetModel : TGetModel;
    GetCube : TGetCube;
    GetDimension : TGetDimension;
    SetElementInfo : TSetElementInfo;
    GetElement : TGetElement;
    SetDataInfo : TSetDataInfo;
    GetData : TGetData;
    GetDataItems : TGetDataItems;
    GetMultipleData : TGetMultipleData;
    SendData : TSendData;
    PutDataItems : TPutDataItems;
    GetAttribute : TGetAttribute;
    GetBasicData: TGetBasicData;
    LoginList : TLoginList;

const LibraryLoaded : Boolean = False;

// -----------------------------------------------------------------------------------------------------------------
Procedure MikAPIExit;
begin
  if hndAPI <> 0 then
  begin
    While LoginList.count > 0 do moLogout(LoginList.Items[0].User, LoginList.Items[0].dbId);
    FreeLibrary(hndAPI);
  end;
  ExitProc := ExitSave;
end;

// -----------------------------------------------------------------------------------------------------------------
Function InitAPI(s: String): Integer;
begin
  if LibraryLoaded then FreeLibrary(hndApi);
  SetCurrentDir(MikOlapPath);
  hndApi := LoadLibrary(PCHAR(MikOlapAPI));
  if hndAPI = 0 then result := -1 else
  result := 0;
  ExitSave := ExitProc;
  ExitProc := @MikAPIExit;
  LoginList := TLoginList.Create;
end;

// -----------------------------------------------------------------------------------------------------------------
function moGetDatabase (UserNr: LongInt;databasid : plong; databasename : PChar; FIRST: LongInt): LongInt;
begin
  if hndAPI = 0 then InitAPI(MikOlapAPI);
  if hndAPI <> 0 then
  begin
    if Pointer(GetDatabase) = NIL then Pointer(GetDatabase) := GetProcAddress(hndAPI, 'GetDatabase');
    if Pointer(GetDatabase) <> NIL then
    begin
      Result := GetDatabase(UserNr,databasid, databasename, FIRST);
    end
    else
      result := moProcNotFound;
  end
  else
    result := moAPINotFound;
end;

// -----------------------------------------------------------------------------------------------------------------
function moGetRequiredLoginParams (Databaseid : LongInt): LongInt;
begin
  if hndAPI = 0 then InitAPI(MikOlapAPI);
  if hndAPI <> 0 then
  begin
    if Pointer(GetRequiredLoginParams) = NIL then Pointer(GetRequiredLoginParams) := GetProcAddress(hndAPI, 'GetRequiredLoginParams');
    if Pointer(GetRequiredLoginParams) <> NIL then
    begin
      Result := GetRequiredLoginParams(DataBaseID);
    end
    else
      result := moProcNotFound;
  end
  else
    result := moAPINotFound;
end;

// -----------------------------------------------------------------------------------------------------------------
function moLogin (Caller: LongInt; loginmode: LongInt; DataBaseID: LongInt; Database: PChar; Name: PChar; Password: PChar; Language: LongInt): LongInt;
begin
  if hndAPI = 0 then InitAPI(MikOlapAPI);
  if hndAPI <> 0 then
  begin
    if Pointer(Login) = NIL then Pointer(Login) := GetProcAddress(hndAPI, 'Login');
    if Pointer(Login) <> NIL then
    begin
      Result := Login(Caller,loginmode,DataBaseID,Database,Name,Password,Language);
      if Result > 0 then LoginList.AddUser(Result,DataBaseID);
    end
    else
      result := moProcNotFound;
  end
  else
    result := moAPINotFound;
end;

// -----------------------------------------------------------------------------------------------------------------
function moGetModel               (UserNr: LongInt;databasid : plong;modid: plong;
                                   modtext: PChar; FIRST: LongInt): LongInt;
begin
  if hndAPI = 0 then InitAPI(MikOlapAPI);
  if hndAPI <> 0 then
  begin
    if Pointer(GetModel) = NIL then Pointer(GetModel) := GetProcAddress(hndAPI, 'GetModel');
    if Pointer(GetModel) <> NIL then
    begin
      Result := GetModel(UserNr,databasid,modid,modtext,FIRST);
    end
    else
      result := moProcNotFound;
  end
  else
    result := moAPINotFound;
end;

// -----------------------------------------------------------------------------------------------------------------
function moLogout (ActUser: LongInt; DatabaseID: LongInt): LongInt;
begin
  if hndAPI = 0 then InitAPI(MikOlapAPI);
  if hndAPI <> 0 then
  begin
    if Pointer(Logout) = NIL then Pointer(Logout) := GetProcAddress(hndAPI, 'Logout');
    if Pointer(Logout) <> NIL then
    begin
      Result := Logout(ActUser, DatabaseID);
      LoginList.DeleteUser(ActUser);
    end
    else
      result := moProcNotFound;
  end
  else
    result := moAPINotFound;
end;

// -----------------------------------------------------------------------------------------------------------------
function moGetCube (ActUser: LongInt; DatabaseID: LongInt; CubeId: pLong; CubeType: pLong; CubeText: pString; FIRST: LongInt): LongInt;
begin
  if hndAPI = 0 then InitAPI(MikOlapAPI);
  if hndAPI <> 0 then
  begin
    if Pointer(GetCube) = NIL then Pointer(GetCube) := GetProcAddress(hndAPI, 'GetCube');
    if Pointer(GetCube) <> NIL then
    begin
      Result := GetCube (ActUser, DatabaseID, CubeId, CubeType, CubeText, FIRST);
    end
    else
      result := moProcNotFound;
  end
  else
    result := moAPINotFound;
end;

// -----------------------------------------------------------------------------------------------------------------
function moGetDimension (ActUser: LongInt; DatabaseID: LongInt; CubeID: LongInt; DimId: pLong; DimType: pLong;
                         DimText: pString; FIRST: LongInt): LongInt;
begin
  if hndAPI = 0 then InitAPI(MikOlapAPI);
  if hndAPI <> 0 then
  begin
    if Pointer(GetDimension) = NIL then Pointer(GetDimension):= GetProcAddress(hndAPI, 'GetDimension');
    if Pointer(GetDimension) <> NIL then
    begin
      Result := GetDimension (ActUser, DatabaseID, CubeID, DimId, DimType, DimText, FIRST);
    end
    else
      result := moProcNotFound;
  end
  else
    result := moAPINotFound;
end;

// -----------------------------------------------------------------------------------------------------------------
function moSetElementInfo (ActUser: LongInt; DatabaseID: LongInt; CubeID: LongInt;
                           DimID: LongInt; HierID: LongInt; Condition: pParams): LongInt;
begin
  if hndAPI = 0 then InitAPI(MikOlapAPI);
  if hndAPI <> 0 then
  begin
    if Pointer(SetElementInfo) = NIL then Pointer(SetElementInfo):= GetProcAddress(hndAPI, 'SetElementInfo');
    if Pointer(SetElementInfo) <> NIL then
    begin
      Result := SetElementInfo (ActUser, DatabaseID, CubeID, DimID, HierID, Condition);
    end
    else
      result := moProcNotFound;
  end
  else
    result := moAPINotFound;
end;

// -----------------------------------------------------------------------------------------------------------------
function moGetElement  (ActUser: LongInt; Info: pParams): LongInt;
begin
  if hndAPI = 0 then InitAPI(MikOlapAPI);
  if hndAPI <> 0 then
  begin
    if Pointer(GetElement) = NIL then Pointer(GetElement) := GetProcAddress(hndAPI, 'GetElement');
    if Pointer(GetElement) <> NIL then
    begin
      Result := GetElement (ActUser, Info);
    end
    else
      result := moProcNotFound;
  end
  else
    result := moAPINotFound;
end;

// -----------------------------------------------------------------------------------------------------------------
function moSetDataInfo (ActUser: LongInt; DatabaseID: LongInt; CubeID: LongInt;
                        DimID: LongInt; HierID: LongInt; Condition: pParams): LongInt;
begin
  if hndAPI = 0 then InitAPI(MikOlapAPI);
  if hndAPI <> 0 then
  begin
    if Pointer(SetDataInfo) = NIL then Pointer(SetDataInfo) := GetProcAddress(hndAPI, 'SetDataInfo');
    if Pointer(SetDataInfo) <> NIL then
    begin
      Result := SetDataInfo (ActUser, DatabaseID, CubeID, DimID, HierID, Condition);
    end
    else
      result := moProcNotFound;
  end
  else
    result := moAPINotFound;
end;

// -----------------------------------------------------------------------------------------------------------------
function moGetData  (ActUser: LongInt; Info: pParams): LongInt;
begin
  if hndAPI = 0 then InitAPI(MikOlapAPI);
  if hndAPI <> 0 then
  begin
    if Pointer(GetData) = NIL then Pointer(GetData) := GetProcAddress(hndAPI, 'GetData');
    if Pointer(GetData) <> NIL then
    begin
      Result := GetData (ActUser, Info);
    end
    else
      result := moProcNotFound;
  end
  else
    result := moAPINotFound;
end;

// -----------------------------------------------------------------------------------------------------------------
function moGetDataItems  (ActUser: LongInt; cubeid: LongInt; options: LongInt;
                           dataitems: pdataitems; Info: ppParams): LongInt;
begin
  if hndAPI = 0 then InitAPI(MikOlapAPI);
  if hndAPI <> 0 then
  begin
    if Pointer(GetDataItems) = NIL then Pointer(GetDataItems) := GetProcAddress(hndAPI, 'GetDataItems');
    if Pointer(GetDataItems) <> NIL then
    begin
      Result := GetDataItems (ActUser, cubeid, options, dataitems, Info);
    end
    else
      result := moProcNotFound;
  end
  else
    result := moAPINotFound;
end;

// -----------------------------------------------------------------------------------------------------------------
function moGetMultipleData (ActUser: LongInt; cubeid: LongInt; options: LongInt;
                             basic: LongInt; dataitems: pDataItems; info: ppParams;
                             FIRST: LongInt): LongInt;
begin
  if hndAPI = 0 then InitAPI(MikOlapAPI);
  if hndAPI <> 0 then
  begin
    if Pointer(GetMultipleData) = NIL then Pointer(GetMultipleData) := GetProcAddress(hndAPI, 'GetMultipleData');
    if Pointer(GetMultipleData) <> NIL then
    begin
      Result := GetMultipleData (ActUser, cubeid, options, basic, dataitems, info, FIRST);
    end
    else
      result := moProcNotFound;
  end
  else
    result := moAPINotFound;
end;

// -----------------------------------------------------------------------------------------------------------------
function moSendData  (ActUser: LongInt; Info: pParams): LongInt;
begin
  if hndAPI = 0 then InitAPI(MikOlapAPI);
  if hndAPI <> 0 then
  begin
    if Pointer(SendData) = NIL then Pointer(SendData) := GetProcAddress(hndAPI, 'SendData');
    if Pointer(SendData) <> NIL then
    begin
      Result := SendData (ActUser, Info);
    end
    else
      result := moProcNotFound;
  end
  else
    result := moAPINotFound;
end;

// -----------------------------------------------------------------------------------------------------------------
function moPutDataItems  (ActUser: LongInt; cubeid: LongInt; options: LongInt;
                           dataitems: pDataItems; Info: ppParams): LongInt;
begin
  if hndAPI = 0 then InitAPI(MikOlapAPI);
  if hndAPI <> 0 then
  begin
    if Pointer(PutDataItems) = NIL then Pointer(PutDataItems) := GetProcAddress(hndAPI, 'PutDataItems');
    if Pointer(PutDataItems) <> NIL then
    begin
      Result := PutDataItems (ActUser, cubeid, options, dataitems, Info);
    end
    else
      result := moProcNotFound;
  end
  else
    result := moAPINotFound;
end;

// -----------------------------------------------------------------------------------------------------------------
function moGetAttribute (ActUser: LongInt; DatabaseID: LongInt; dimid: LongInt;
                          elementname: pString; attrdimid: LongInt; resulttext: pString;
                          FIRST: LongInt): LongInt;
begin
  if hndAPI = 0 then InitAPI(MikOlapAPI);
  if hndAPI <> 0 then
  begin
    if Pointer(GetAttribute) = NIL then Pointer(GetAttribute) := GetProcAddress(hndAPI, 'GetAttribute');
    if Pointer(GetAttribute) <> NIL then
    begin
      Result := GetAttribute (ActUser, DatabaseID, dimid, elementname, attrdimid, resulttext, FIRST);
    end
    else
      result := moProcNotFound;
  end
  else
    result := moAPINotFound;
end;

// -----------------------------------------------------------------------------------------------------------------
function moGetBasicData ( ActUser: LONGINT; DatabaseID: LONGINT; dataitems: pDataitems;
                           Info: ppParams; FIRST: LONGINT ): LONGINT;
begin
  if hndAPI = 0 then InitAPI(MikOlapAPI);
  if hndAPI <> 0 then
  begin
    if Pointer(GetBasicData) = NIL then Pointer(GetBasicData) := GetProcAddress(hndAPI, 'GetBasicData');
    if Pointer(GetBasicData) <> NIL then
    begin
      Result := GetBasicData (ActUser, DatabaseID, dataitems, Info, FIRST);
    end
    else
      result := moProcNotFound;
  end
  else
    result := moAPINotFound;
end;

function  TLogInList.GetUser(i: Integer): TLoginItem;
var fInfo : ^TLoginItem;
begin
  fInfo := inherited Items[i];
  Result := fInfo^;
end;

function  TLogInList.AddUser(User, dbId: LongInt): Integer;
var fInfo : ^TLoginItem;
begin
  new(fInfo);
  fInfo^.User := User;
  fInfo^.dbId := dbId;
  inherited Add(fInfo);
  result := count-1;
end;

Procedure TLogInList.DeleteUser(User: LongInt);
var i : Integer;
    boFound : Boolean;
    fInfo : ^TLoginItem;
begin
  FillChar(fInfo,SizeOf(fInfo),0);
  i := 0;
  boFound := False;
  if count > 0 then
  begin
    while not boFound do
    begin
      fInfo := inherited Items[i];
      if fInfo^.User = User then boFound := True
      else                   inc(i);
      if i = count then Exit;
    end;
    if boFound then
    begin
      Dispose(fInfo);
      Delete(i);
    end;
  end;
end;

end.
