unit MikDataLst;

interface

uses
  Windows, Messages, SysUtils, Classes, MikTypes, LResources;

type
  TOlapData   = class;
  TDataSerien = class;
  TMikDataLst = class;

  TDataInfo = Record
                level  : LongInt;
                Value  : Double;
                Text   : String;
              end;

  TTrendInfo = record
     TrendTyp   : TTrendType;                                                   // ttLinear, ttSectionAverage,ttFloatingAverage,ttSummary,ttNone
     StartIndex : Integer;                                                      //  Linear, Periodenschnitt, Gleitender Durchscn., Summe, Keines
     EndIndex   : Integer;
     RefSerie1  : Integer;
     RefSerie2  : Integer;
     DataIndex  : Integer;
  end;

  TSeriesInfo = record
     SerieTyp   : TSerieType;                                                   // tstOlapData, tstTrenddata
     TrendInfo  : TTrendInfo;
     Legend     : String;
  end;

  TOlapData  = Class(TList)
  private     // Private-Deklarationen
    fData       : ^TDataInfo;
    fSeriesInfo : TSeriesInfo;
  protected   // Protected-Deklarationen
    function    GetData(Index : Integer): TDataInfo;
    Procedure   SetData(Index : Integer; Data : TDataInfo);
    procedure   clearList;
  public      // Public-Deklarationen
    SeriesInfo : TSeriesInfo;
    Constructor Create;
    destructor  Destroy; override;
    function    AddData(level : LongInt;Value : Double;Text  : String): Integer;
    Procedure   SetValue(Index : Integer; Value: Double);
    Function    GetValue(Index : Integer): Double;
    Function    GetText(Index : Integer): String;
    Function    GetLevel(Index : Integer): LongInt;
    Function    DeleteData(Index : Integer): Integer;
    property    Data[Index : Integer]: TDataInfo read GetData;
  end;

  TDataSerien = Class(TList)
  private
    fSerie : TOlapData;
  protected
    function  GetSerie(Index : Integer): TOlapData;
  public
    destructor Destroy; override;
    procedure  Clear; override;
    Function   DeleteSerie(Index : Integer): Integer;
    Function   AddSerie: Integer;
    property   Serie[Index : Integer]: TOlapData read GetSerie;
  end;

  TMikDataLst = class(TComponent)
  private
    // Private-Deklarationen
  protected
    // Protected-Deklarationen
    function    TrendAnfangswert(List: TOlapData): double;
    function    TrendLineareVeraenderung(List: TOlapData): double;
    function    TrendGdUngerade(Index: Integer; Periodenschnitt: Integer): Integer;
    function    TrendGdGerade(Index: Integer; Periodenschnitt: Integer): Integer;
  public
    // Public-Deklarationen
    DataSeries : TDataSerien;
    Constructor Create(AOwner: TComponent); override;
    Destructor  Destroy; override;
    Procedure   ClearSerieslist;
    Function    TrendLinear(Index: Integer; iCount: Integer): Integer;
    Function    TrendFloatingAverage(Index: Integer; Periodenschnitt: Integer): Integer; //gleitender Periodendurchschnitt
    function    TrendPeriodAverage(Index: Integer; Periodenschnitt: Integer): Integer;   //Periodenschnitt
    Function    IndexTo(IndexValue, IndexReferenz, Faktor: Integer): Integer;
    Function    Mean(iSeriesValue, iSeriesCount: Integer) : Double;                      //gewogener arithmetischer Mittelwert einer Serie
    Function    Median(iSeries: Integer): Double;                                        //Mittelwert, Wert in der Mitte eine aufsteigend sortierten Serie
    Function    Mode(Index: Integer): TModeType;                                         //Modus-hufigster Wert in einer Serie
    Function    Smallest(Index: Integer): Double;
    Function    Biggest(Index: Integer): Double;
    Function    Spread(Index: Integer) : Double;                                         //Spannweite
    Function    Divergence(iSeriesValue, iSeriesCount: Integer) : Double;                //durchschnittliche mittlere Abweichung
    Function    Multiply(iSeriesValue, iSeriesCount: Integer) : Integer;                 //Produkt aus zwei Serien
    Function    AddToSeries(iSeriesTarget, iSeriesValue: Integer) : Integer;             //addiert ValueSerie zu TargetSerie
    Function    VariationsKoeffizient(iSeriesValue, iSeriesCount: Integer) : Double;
    Function    Kumulation(Index: Integer) : Integer;                                    //kumulierte Serie
  published
    // Published-Deklarationen
  end;

procedure Register;

implementation

//------------------------------------------------------------------------------
//Methoden fr TMikDataLst
//------------------------------------------------------------------------------

constructor TMikDataLst.Create(AOwner: TComponent);
begin
  inherited create(AOwner);
  DataSeries := TDataSerien.Create;
end;

//------------------------------------------------------------------------------
Destructor  TMikDataLst.Destroy;
begin
  Dataseries.Free;
  inherited Destroy;
end;

//------------------------------------------------------------------------------
//Function   TMikDataLst.NewSerie;
//begin
//  result := DataSeries.AddSerie;
//end;

//------------------------------------------------------------------------------
Procedure   TMikDataLst.ClearSerieslist;
begin
  While DataSeries.count > 0 do
        DataSeries.Clear;
end;

//------------------------------------------------------------------------------
function TMikDataLst.TrendAnfangswert(List: TOlapData): double;
var i : Integer;
    sXi, sYi, sXiYi, sXiXi : double;
begin
  sXi := 0.0;
  sYi := 0.0;
  sXiYi := 0.0;
  sXiXi := 0.0;
  // sXi    for i := 0 to List.count-1 do sXi := sXi + i;
  // sYi    for i := 0 to List.count-1 do sYi := sYi + List.GetValue(i);
  // sXiYi  for i := 0 to List.count-1 do sXiYi := sXiYi + (List.GetValue(i)*i);
            For i := 0 to List.count-1 do sXiXi := sXiXi + (i*i);

  TrendAnfangswert := ( ((sXiXi*sYi) - (sXi * sXiYi)) / ((List.count * sXiXi)- (sXi * sXi)) );
end;

//------------------------------------------------------------------------------
function TMikDataLst.TrendLineareVeraenderung(List: TOlapData): double;
var i : Integer;
    sXi, sYi, sXiYi, sXiXi : double;
begin
  sXi := 0.0;
  sYi := 0.0;
  sXiYi := 0.0;
  sXiXi := 0.0;
  // sXi    for i := 0 to List.count-1 do sXi := sXi + i;
  // sYi    for i := 0 to List.count-1 do sYi := sYi + List.GetValue(i);
  // sXiYi  for i := 0 to List.count-1 do sXiYi := sXiYi + (List.GetValue(i)*i);
            For i := 0 to List.count-1 do sXiXi := sXiXi + (i*i);

  TrendLineareVeraenderung := ((sXi * sYi) - (List.count * sXiYi)) / ( (sXi * sXi) - (List.count * sXiXi) );

end;

//------------------------------------------------------------------------------
Function  TMikDataLst.TrendLinear(Index: Integer; iCount: Integer): Integer;
var i, iSerie : Integer;
    dStart, dDiff : Double;
begin
{
  result := -1;
  if (Index > -1) and (Index < DataSeries.count) then
  begin
    dStart := TrendAnfangswert(DataSeries.Items[Index]);
    dDiff  := TrendLineareVeraenderung(DataSeries.Items[Index]);
    iSerie := DataSeries.AddSerie;
    DataSeries.Serie[iSerie].fSeriesInfo.Legend := 'Linear '+DataSeries.Serie[Index].fSeriesInfo.Legend;
    DataSeries.Serie[iSerie].fSeriesInfo.SerieTyp := tstTrendData;
    DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.TrendTyp := ttLinear;
    DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.DataIndex := iSerie;
    Result := iSerie;
    For i := 0 to DataSeries.Serie[Index].count-1 do
    begin
       DataSeries.Serie[iSerie].AddData(0,dStart,'');
       dStart := dStart+dDiff;
    end;
  end;
}
end;

//------------------------------------------------------------------------------
Function    TMikDataLst.IndexTo(IndexValue, IndexReferenz, Faktor: Integer): Integer;
var iData : Integer;
    iSerie : Integer;
begin
  result := -1;
  Try
    iSerie := DataSeries.AddSerie;
    DataSeries.Serie[iSerie].fSeriesInfo.Legend := 'Index '+DataSeries.Serie[IndexValue].fSeriesInfo.Legend;
    DataSeries.Serie[iSerie].fSeriesInfo.SerieTyp := tstTrendData;
    DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.TrendTyp := ttIndexTo;
    DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.RefSerie1  := IndexValue;
    DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.RefSerie2  := IndexReferenz;
    DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.StartIndex := 0;
    For iData := 0 to Dataseries.Serie[IndexValue].count-1 do
    begin
      DataSeries.Serie[iSerie].AddData(0,DataSeries.Serie[IndexValue].Data[iData].Value * Faktor / DataSeries.Serie[IndexReferenz].Data[iData].Value,'');
      DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.EndIndex   := iData;
    end;
    Result := iSerie;
  except
  end; //Try
end;

//------------------------------------------------------------------------------
Function    TMikDataLst.TrendFloatingAverage(Index: Integer; Periodenschnitt: Integer): Integer;
begin
  if Periodenschnitt mod 2 > 0 then
  begin
     result := TrendGdUngerade(Index,PeriodenSchnitt);
  end
  else
     result := TrendGdGerade(Index,PeriodenSchnitt);
end;

//------------------------------------------------------------------------------
function  TMikDataLst.TrendGdUngerade(Index: Integer; Periodenschnitt: Integer): Integer;
var iArea    : Integer; //Bereichsvariable fr Trendwert
    iData    : Integer; //Zeiger auf Trendposition
    Value    : Double;
    i, iDataStart, iDataEnde, iSerie : Integer;
begin
  result := -1;
  if (Index > -1) and (Index < DataSeries.count) then
  begin
Try
    iArea := PeriodenSchnitt DIV 2;
    iDataStart := iArea+1;
    iDataEnde  := Dataseries.Serie[Index].count-iArea-2;
    iSerie := DataSeries.AddSerie;
    DataSeries.Serie[iSerie].fSeriesInfo.Legend := 'gleitender Durchschnitt '+DataSeries.Serie[Index].fSeriesInfo.Legend;
    DataSeries.Serie[iSerie].fSeriesInfo.SerieTyp := tstTrendData;
    DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.TrendTyp := ttFloatingAverage;
    DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.RefSerie1  := Index;
    DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.RefSerie2  := -1;
    DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.StartIndex := iDataStart;
    for i := 0 to iDataStart-1 do DataSeries.Serie[iSerie].AddData(0,0.0,'');
    For iData := iDataStart to iDataEnde do
    begin
      Value := 0.0;
      For i := (iData-iArea) to (iData+iArea) do Value := Value + DataSeries.Serie[Index].Data[i].Value;
      DataSeries.Serie[iSerie].AddData(0,Value / Periodenschnitt,'');
      DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.EndIndex   := iData;
    end;
    for i := iDataEnde+1 to Dataseries.Serie[Index].count do DataSeries.Serie[iSerie].AddData(0,0.0,'');
    Result := iSerie;
except
end;
  end;
end;

//------------------------------------------------------------------------------
function  TMikDataLst.TrendGdGerade(Index: Integer; Periodenschnitt: Integer): Integer;
var iArea    : Integer; //Bereichsvariable fr Trendwert
    iData    : Integer; //Zeiger auf Trendposition
    Value1, Value2    : Double;
    i, iDataStart, iDataEnde, iSerie : Integer;
begin
  result := -1;
  if (Index > -1) and (Index < DataSeries.count) then
  begin
try
    iArea := PeriodenSchnitt DIV 2;
    iDataStart := iArea+1;
    iDataEnde  := Dataseries.Serie[Index].count-iArea-1;
    Value1 := 0.0;
    Value2 := 0.0;
    iSerie := DataSeries.AddSerie;
    DataSeries.Serie[iSerie].fSeriesInfo.Legend := 'gleitender Durchschnitt '+DataSeries.Serie[Index].fSeriesInfo.Legend;
    DataSeries.Serie[iSerie].fSeriesInfo.SerieTyp := tstTrendData;
    DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.TrendTyp := ttFloatingAverage;
    DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.DataIndex := iSerie;
    DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.StartIndex := iDataStart;
    DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.RefSerie1  := Index;
    DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.RefSerie2  := -1;
    for iData := 0 to iDataStart-1 do DataSeries.Serie[iSerie].AddData(0,0.0,'');
    For iData := iDataStart to iDataEnde-1 do
    begin
      if Value1 = 0.0 then For i := (iData-iArea-1) to (iData+iArea-1) do Value1 := Value1 + DataSeries.Serie[Index].Data[i].Value;
      For i := (iData-iArea)   to (iData+iArea)   do Value2 := Value2 + DataSeries.Serie[Index].Data[i].Value;
      Value1 := Value1 / Periodenschnitt;
      Value2 := Value2 / Periodenschnitt;
      DataSeries.Serie[iSerie].AddData(0,(Value1 + Value2) / 2,'');
      DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.EndIndex   := iData;
      Value1 := Value2;
    end;
    for iData := iDataEnde+1 to Dataseries.Serie[Index].count do DataSeries.Serie[iSerie].AddData(0,0.0,'');
    Result := iSerie;
except
end;
  end;
end;

//------------------------------------------------------------------------------
function TMikDataLst.TrendPeriodAverage(Index: Integer; Periodenschnitt: Integer): Integer;
var iArea    : Integer; //Bereichsvariable fr Trendwert
    iData    : Integer; //Zeiger auf Trendposition
    Value    : Double;
    i, iDataStart, iDataEnde, iSerie : Integer;
begin
  iArea := PeriodenSchnitt;
  iDataStart := iArea;
  iDataEnde  := Dataseries.Serie[Index].count-1;
  iSerie := DataSeries.AddSerie;
  DataSeries.Serie[iSerie].fSeriesInfo.Legend := 'Periodenschnitt '+DataSeries.Serie[Index].fSeriesInfo.Legend;
  DataSeries.Serie[iSerie].fSeriesInfo.SerieTyp := tstTrendData;
  DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.TrendTyp   := ttSectionAverage;
  DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.DataIndex  := iSerie;
  DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.StartIndex := iDataStart;
  DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.RefSerie1  := Index;
  DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.RefSerie2  := -1;
  For i := 0 to iDataStart-1 do DataSeries.Serie[iSerie].AddData(0,0,'');
  For iData := iDataStart to iDataEnde do
  begin
    Value := 0.0;
    For i := (iData-iArea) to iData do
        Value := Value + DataSeries.Serie[Index].Data[i].Value;
    DataSeries.Serie[iSerie].AddData(0,Value / Periodenschnitt,'');
    DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.EndIndex   := iData;
    //DataSeries.Serie[Index].SetTrendValue(i,Value / Periodenschnitt);
  end;
  result := iSerie;
end;

// gewogener arithmetischer Mittelwert einer Serie -----------------------------
Function    TMikDataLst.Mean(iSeriesValue, iSeriesCount: Integer) : Double;
var sumValue : Double;
    sumCount : Double;
    i : Integer;
begin
  sumValue := 0.0;
  sumCount := 0.0;
  if iSeriesCount > -1 then
  begin
    For i := 0 to Dataseries.Serie[iSeriesValue].count-1 do
       sumValue := sumValue + (Dataseries.Serie[iSeriesValue].GetValue(i) * Dataseries.Serie[iSeriesCount].GetValue(i));
  end
  else
  begin
    For i := 0 to Dataseries.Serie[iSeriesValue].count-1 do
       sumValue := sumValue + (Dataseries.Serie[iSeriesValue].GetValue(i));
  end;
  if iSeriesCount > -1 then
  begin
     For i := 0 to Dataseries.Serie[iSeriesValue].count-1 do
        sumCount := sumCount + Dataseries.Serie[iSeriesCount].GetValue(i);
  end
  else
  begin
    sumCount := Dataseries.Serie[iSeriesValue].count;
  end;

  Result := sumValue / sumCount;
end;


//------------------------------------------------------------------------------
Function SortByValue(Item1,Item2: Pointer): Integer;
begin
  if (Double(Item1^) < Double(Item2^)) then result := -1
  else
     if (Double(Item1^) > Double(Item2^)) then result := 1
     else
        result := 0;
end;

// Zentalwert ------------------------------------------------------------------
Function    TMikDataLst.Median(iSeries: Integer): Double;
var tli : TList;
    tliValue : ^double;
    i : Integer;
begin
  tli := Tlist.Create;
  For i := 0 to Dataseries.Serie[iSeries].count-1 do
  begin
    New(tliValue);
    tliValue^ := Dataseries.Serie[iSeries].GetValue(i);
    tli.Add(tliValue);
  end;
  tli.Sort(@SortByValue);
  if (tli.Count mod 2) > 0 then
     i := (tli.Count div 2) + 1
  else
     i := (tli.Count+1) div 2;
  result := Double(tli.Items[i]^);
  while tli.Count > 0 do
  begin
    tliValue := tli.Items[tli.count-1];
    dispose(tliValue);
    tli.Delete(tli.count-1);
  end;
  tli.free;
end;

//------------------------------------------------------------------------------
Function    TMikDataLst.Smallest(Index: Integer): Double;
var tli : TList;
    tliValue : ^double;
    i : Integer;
begin
  result := 0.0;
  if (Index < 0) or (Index < Dataseries.Count) then
  begin
     tli := Tlist.Create;
     For i := 0 to Dataseries.Serie[Index].count-1 do
     begin
       New(tliValue);
       tliValue^ := Dataseries.Serie[Index].GetValue(i);
       tli.Add(tliValue);
     end;
     tli.Sort(@SortByValue);
     tliValue := tli.First;
     Result := tliValue^;
     while tli.Count > 0 do
     begin
       tliValue := tli.Items[tli.count-1];
       dispose(tliValue);
       tli.Delete(tli.count-1);
     end;
     tli.Free;
  end;
end;

//------------------------------------------------------------------------------
Function    TMikDataLst.Biggest(Index: Integer): Double;
var tli : TList;
    tliValue : ^double;
    i : Integer;
begin
  result := 0.0;
  if (Index < 0) or (Index < Dataseries.Count) then
  begin
    tli := Tlist.Create;
    For i := 0 to Dataseries.Serie[Index].count-1 do
    begin
      New(tliValue);
      tliValue^ := Dataseries.Serie[Index].GetValue(i);
      tli.Add(tliValue);
    end;
    tli.Sort(@SortByValue);
    tliValue := tli.Last;
    Result := tliValue^;
    while tli.Count > 0 do
    begin
      tliValue := tli.Items[tli.count-1];
      dispose(tliValue);
      tli.Delete(tli.count-1);
    end;
    tli.free;
  end;
end;

//------------------------------------------------------------------------------
Function InTliResult(tlResult: TList;v : Double): Integer;
var boNotEnde: Boolean;
    iFind : Integer;
    s : ^TModeType;
begin
  result := -1;
  iFind := -1;
  if (tlResult.Count = 0) then
  begin
    result := -1;
    exit;
  end;
  boNotEnde := (tlResult.Count > 0);
  while boNotEnde do
  begin
    inc(iFind);
    if iFind <= tlResult.Count-1 then
    begin
      s := tlResult.Items[iFind];
      if s^.Value = v then
      begin
        result := iFind;
        boNotEnde := False;
      end;
    end
    else
      boNotEnde := False;
  end;
end;

//------------------------------------------------------------------------------
Function SortByValue2(Item1,Item2: Pointer): Integer;
var s1, s2 : ^TModeType;
begin
  s1 := Item1;
  s2 := Item2;
  if (s1^.count < s2^.count) then result := -1
  else
     if (s1^.count > s2^.count) then result := 1
     else
        result := 0;
end;

//------------------------------------------------------------------------------
Function    TMikDataLst.Mode(Index: Integer): TModeType;
var tli, tliResult : TList;
    tliValue, tliFind : ^double;
    tmtFound : ^TModeType;
    i,iFound : Integer;
begin
  result.value := 0.0;
  result.count := 0;
try
  if (Index < 0) or (Index < Dataseries.Count) then
  begin
    tli := Tlist.Create;
    tliResult := Tlist.Create;
    For i := 0 to Dataseries.Serie[Index].count-1 do
    begin
      New(tliValue);
      tliValue^ := Dataseries.Serie[Index].GetValue(i);
      tli.Add(tliValue);
    end;

    new(tliFind);
    For i := 0 to tli.Count-1 do
    begin
      tliFind := tli.Items[i];
      iFound := InTliResult(tliResult,Double(tliFind^));
      if iFound = -1 then
      begin
        new(tmtFound);
        tmtFound^.value := tliFind^;
        tmtFound^.count := 1;
        tliResult.Add(tmtFound);
      end
      else
      begin
        TModeType(tliResult.Items[iFound]^).Count := TModeType(tliResult.Items[iFound]^).Count+1;
      end;
    end;

    if tliResult.Count > 0 then
    begin
      tliResult.Sort(@SortByValue2);
      tmtFound := tliResult.Items[tliResult.Count-1];
      if tmtFound^.count > 1 then
         result := TModeType(tmtFound^);
    end;

    while tli.Count > 0 do
    begin
      tliValue := tli.Items[0];
      if tliValue <> nil then dispose(tliValue);
      tli.Delete(0);
    end;

    while tliResult.Count > 0 do
    begin
      //tmtFound := tliResult.Items[0];
      //if tmtFound <> nil then dispose(tmtFound);
      tliResult.Delete(0);
    end;
    //if tliFind <> NIL then Dispose(tliFind);
    tli.free;
    tliResult.Free;
  end;
except
end; //try
end;

//------------------------------------------------------------------------------
Function    TMikDataLst.Spread(Index: Integer) : Double;
var tli : TList;
    tliBiggest, tliSmallest : ^double;
    i : Integer;
begin
  result := 0.0;
  if (Index < 0) or (Index < Dataseries.Count) then
  begin
     tli := Tlist.Create;
     For i := 0 to Dataseries.Serie[Index].count-1 do
     begin
       New(tliBiggest);
       tliBiggest^ := Dataseries.Serie[Index].GetValue(i);
       tli.Add(tliBiggest);
     end;
     tli.Sort(@SortByValue);
     tliBiggest := tli.Last;
     tliSmallest := tli.First;
     Result := tliBiggest^ -tliSmallest^;
     while tli.Count > 0 do
     begin
       tliBiggest := tli.Items[tli.count-1];
       dispose(tliBiggest);
       tli.Delete(tli.count-1);
     end;
     tli.Free;
  end;
end;

//------------------------------------------------------------------------------
Function    TMikDataLst.Divergence(iSeriesValue, iSeriesCount: Integer) : Double;  //durchschnittliche mittlere Abweichung
var xMean : Double;
    i : Integer;
    dSumme : Double;
    dCount : Double;
begin
  dSumme := 0.0;
  dCount := 0.0;
  xMean := Mean(iSeriesValue, iSeriesCount);
  if iSeriesCount > -1 then
  begin
     For i := 0 to Dataseries.Serie[iSeriesValue].count-1 do
     begin
         dSumme := dSumme + abs(Dataseries.Serie[iSeriesValue].GetValue(i)-xMean)*Dataseries.Serie[iSeriesCount].GetValue(i);
         dCount := dCount + Dataseries.Serie[iSeriesCount].GetValue(i);
     end;
  end
  else
  begin
     For i := 0 to Dataseries.Serie[iSeriesValue].count-1 do
     begin
         dSumme := dSumme + abs(Dataseries.Serie[iSeriesValue].GetValue(i)-xMean);
         dCount := dCount + 1;
     end;
  end;
  result := dSumme/dCount;
// ohne Vorzweichen ( Wert - gewogenes Mittel) / Anzahl
end;

//------------------------------------------------------------------------------
Function    TMikDataLst.VariationsKoeffizient(iSeriesValue, iSeriesCount: Integer) : Double;
var StdAbw, Mittel: Double;
begin
  StdAbw := Divergence(iSeriesValue, iSeriesCount);
  Mittel := Mean(iSeriesValue, iSeriesCount);
  Result := stdAbw*100/Mittel;
end;

//------------------------------------------------------------------------------
Function    TMikDataLst.Multiply(iSeriesValue, iSeriesCount: Integer) : Integer; //Bildet Summe aus zwei Serien in neue Serie, Rckgabe Index der Serie
var iData : Integer;
    iSerie : Integer;
begin
  result := -1;
  Try
    iSerie := DataSeries.AddSerie;
    DataSeries.Serie[iSerie].fSeriesInfo.Legend := 'Index '+DataSeries.Serie[iSeriesValue].fSeriesInfo.Legend;
    DataSeries.Serie[iSerie].fSeriesInfo.SerieTyp := tstCalculated;
    DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.TrendTyp := ttMultiply;
    DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.StartIndex := 0;
    DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.EndIndex   := Dataseries.Serie[iSeriesValue].count-1;
    DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.RefSerie1  := iSeriesValue;
    DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.RefSerie2  := iSeriesCount;
    For iData := 0 to Dataseries.Serie[iSeriesValue].count-1 do
    begin
      DataSeries.Serie[iSerie].AddData(0,DataSeries.Serie[iSeriesValue].Data[iData].Value * DataSeries.Serie[iSeriesCount].Data[iData].Value,'');
      DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.EndIndex   := iData;
    end;
    Result := iSerie;
  except
  end; //Try
end;

// addiert ValueSerie zu TargetSerie -------------------------------------------
Function    TMikDataLst.AddToSeries(iSeriesTarget, iSeriesValue: Integer) : Integer;
var iData  : Integer;
    iSerie : Integer;
    newValue : Double;
    iMod     : Integer;
begin
  result := -1;
  iMod   := 0;
  Try
    if (iSeriesTarget = -1) or (iSeriesTarget > DataSeries.Count-1) then
    begin
       iSerie := DataSeries.AddSerie;
       iMod := 1;
    end
    else
       iSerie := iSeriesTarget;

    DataSeries.Serie[iSerie].fSeriesInfo.Legend := 'Summe';
    DataSeries.Serie[iSerie].fSeriesInfo.SerieTyp := tstCalculated;
    DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.TrendTyp := ttAdded;
    DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.StartIndex := 0;
    DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.EndIndex   := -1;
    DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.RefSerie1  := -1;
    DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.RefSerie2  := -1;
    For iData := 0 to Dataseries.Serie[iSeriesValue].count-1 do
    begin
      if iMod = 0 then
         NewValue := DataSeries.Serie[iSeriesTarget].Data[iData].Value+DataSeries.Serie[iSeriesValue].Data[iData].Value
      else
         NewValue := DataSeries.Serie[iSeriesValue].Data[iData].Value;
      if iData > DataSeries.Serie[iSerie].Count-1 then
         DataSeries.Serie[iSerie].AddData(0,NewValue,'')
      else
         DataSeries.Serie[iSerie].SetValue(iData,NewValue);
    end;
    Result := iSerie;
  except
  end; //Try
end;

// kumulierte Serie ------------------------------------------------------------
Function    TMikDataLst.Kumulation(Index: Integer) : Integer;
var iData : Integer;
    iSerie : Integer;
begin
  result := -1;
  Try
  if (Index < 0) or (Index < Dataseries.Count) then
  begin
    iSerie := DataSeries.AddSerie;
    Result := iSerie;
    DataSeries.Serie[iSerie].fSeriesInfo.Legend := 'kumuliert '+DataSeries.Serie[Index].fSeriesInfo.Legend;
    DataSeries.Serie[iSerie].fSeriesInfo.SerieTyp := tstCalculated;
    DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.TrendTyp := ttKumulated;
    DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.StartIndex := 0;
    DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.EndIndex   := Dataseries.Serie[Index].count-1;
    DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.RefSerie1  := Index;
    DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.RefSerie2  := -1;
    DataSeries.Serie[iSerie].AddData(0,DataSeries.Serie[Index].Data[0].Value,'');
    For iData := 1 to Dataseries.Serie[Index].count-1 do
    begin
      DataSeries.Serie[iSerie].AddData(0,DataSeries.Serie[Index].Data[iData].Value + DataSeries.Serie[Index].Data[iData-1].Value,'');
      DataSeries.Serie[iSerie].fSeriesInfo.TrendInfo.EndIndex   := iData;
    end;
  end;
  except
  end; //Try
end;
//------------------------------------------------------------------------------
//Methoden fr TDataSeries
//------------------------------------------------------------------------------
Destructor  TDataSerien.Destroy;
begin
  clear;
  inherited Destroy;
end;

//------------------------------------------------------------------------------
Function  TDataSerien.DeleteSerie(Index : Integer): Integer;
begin
  result := -1;
try
  if (Index > -1) and (Index < count) then
  begin
    fSerie.clearList;
    FreeAndNil(fSerie);
//    fSerie := inherited fSerie^.Items[Index];
//    fSerie.clear;
//    fSerie.Free;
    inherited Delete(Index);
    result := Index;
  end;
except
end; //try
end;

//------------------------------------------------------------------------------
procedure TDataSerien.Clear;
begin
  while count > 0 do DeleteSerie(Count-1);
  inherited clear;
end;

//------------------------------------------------------------------------------
function  TDataSerien.AddSerie: Integer;
begin
  result := -1;
try
   fSerie := TOlapData.Create;
   inherited Add(fSerie);
   result := count - 1;
except
end;
end;

//------------------------------------------------------------------------------
function  TDataSerien.GetSerie(Index: Integer): TOlapData;
begin
   result := NIL;
try
   result := GetSerie(Index);
//   result := inherited Items[Index];
except
end;
end;

//------------------------------------------------------------------------------
//Methoden fr TOlapData
//------------------------------------------------------------------------------
Constructor TOlapData.Create;
begin
  inherited create;
end;

//------------------------------------------------------------------------------
Destructor  TOlapData.Destroy;
begin
  Clear;
  inherited Destroy;
end;

//------------------------------------------------------------------------------
procedure TOlapData.clearList;
begin
  while count > 0 do DeleteData(count-1);
  inherited clear;
end;

//------------------------------------------------------------------------------
Function  TOlapData.DeleteData(Index : Integer): Integer;
begin
  result := -1;
  if (Index > -1) and (Index < count) then
  begin
     fData := inherited Items[Index];
     inherited Delete(Index);
     result := 0;
  end;
end;

//------------------------------------------------------------------------------
function  TOlapData.AddData(level : LongInt;Value : Double;Text  : String): Integer;
begin
   new(fData);
   fData^.level := level;
   fData^.Value := Value;
   fData^.Text  := Text;
   inherited Add(fData);
   result := count - 1;
end;

//------------------------------------------------------------------------------
function  TOlapData.GetData(Index: Integer): TDataInfo;
begin
  Result.level  := -1;
  Result.Value  := 0;
  Result.Text   := '';
  if (Index > -1) and (Index < count) then
  begin
     fData := inherited Items[Index];
     Result.level  := fData^.level;
     Result.Value  := fData^.Value;
     Result.Text   := fdata^.Text;
  end;
end;

//------------------------------------------------------------------------------
Procedure TOlapData.SetData(Index : Integer; Data : TDataInfo);
begin
  if (Index > -1) and (Index < count) then
  begin
    fData := inherited Items[Index];
    fData^.level := Data.level;
    fData^.Value := Data.Value;
    fData^.Text  := Data.Text;
  end;
end;

//------------------------------------------------------------------------------
Function  TOlapData.GetValue(Index : Integer): Double;
begin
  result := 0;
  if (Index > -1) and (Index < count) then
  begin
    fData := inherited Items[Index];
    Result := fData^.Value;
  end;
end;

//------------------------------------------------------------------------------
Procedure TOlapData.SetValue(Index : Integer; Value: Double);
begin
  if (Index > -1) and (Index < count) then
  begin
    fData := inherited Items[Index];
    fData^.Value := Value;
  end;
end;

//------------------------------------------------------------------------------
Function  TOlapData.GetText(Index : Integer): String;
begin
  Result := '';
  if (Index > -1) and (Index < count) then
  begin
    fData := inherited Items[Index];
    Result := fData^.Text;
  end;
end;

//------------------------------------------------------------------------------
Function  TOlapData.GetLevel(Index : Integer): LongInt;
begin
  Result := -1;
  if (Index > -1) and (Index < count) then
  begin
    fData := inherited Items[Index];
    Result := fData^.Level;
  end;
end;


procedure Register;
begin
  RegisterComponents('MikData', [TMikDataLst]);
end;

initialization
{$I TMikDatalst.lrs}

end.
