unit MikParameter;

interface

uses
  Windows, Messages, SysUtils, Classes, MikTypes, LResources;

type
  TMikQueryDimension  = class;
  TMikQueryDimensions = class;
  TMikQuerySerie      = class;
  TMikQuerySerien     = Class;
  TMikParameter       = class;

  TParaInfo = Record
    DimensionId : LongInt;
    Element     : String;
    DimOrder    : Integer;
    DimType     : TDimensionTypes;
  end;

  TMikQueryElements = class(TList)
    fElement : ^String;
    fOwner   : TMikQueryDimension;
  private     { Private-Deklarationen }
  protected   { Protected-Deklarationen }
    constructor Create(AOwner: TMikQueryDimension); virtual;
    destructor  Destroy; override;
    Function    GetElement(Index: Integer): String;
    Function    AddElement(Value: String): Integer;
    Function    DeleteElement(Index: Integer): Integer;
    property    Element[i : Integer] : String read GetElement;
  public
  end;

  TMikQueryDimension = class(TPersistent)
  private
    Elements       : TMikQueryElements;
    fDimId         : LongInt;
    fDimType       : TDimensionTypes;
    fDimOrder      : Integer;
    fOwner         : TMikQueryDimensions;
  protected
    constructor Create(AOwner: TMikQueryDimensions); virtual;
    destructor  Destroy; override;
    Function    GetElement(Index: Integer): String;
    Function    GetElementCount: Integer;
  public
    Function    AddElement(Value: String): Integer;
    Function    DeleteElement(Index: Integer): Integer;
    property    Element[i : Integer] : String read GetElement;
    Property    ElementCount : Integer read GetElementCount;
    property    DimensionOrder : Integer read fDimOrder;
    property    DimensionId : LongInt read fDimId;
    property    DimensionTyp : TDimensionTypes read fDimType;
  end;

  TMikQueryDimensions = class(TList)
  private     { Private-Deklarationen }
    fDimension    : TMikQueryDimension;
    fOwner        : TMikQuerySerie;
    fAktDimension : Integer;
  protected   { Protected-Deklarationen }
    constructor Create(AOwner: TMikQuerySerie); virtual;
    destructor  Destroy; override;
    Function    GetDimension(Index : Integer): TMikQueryDimension;
    Function    AddDimension(DimId: LongInt; DimType : TDimensionTypes; DimOrder: Integer): Integer;
    Function    DeleteDimension(Index: Integer): Integer;
    Function    GetDimensionByOrder(DimOrder : Integer): TMikQueryDimension;
    property    Dimension[i: Integer] : TMikQueryDimension read GetDimension;
  public
  end;

  TMikQuerySerie = class(TPersistent)
  private     { Private-Deklarationen }
    Dimensions    : TMikQueryDimensions;
    fCubeId       : LongInt;
    fValue        : double;
    fMultiElement : Integer;
    fOwner        : TList;
    fLegende      : String;
  protected   { Protected-Deklarationen }
    constructor Create(AOwner: TList); virtual;
    destructor  Destroy; override;
    Function    GetDimension(Index : Integer): TMikQueryDimension;
    Function    GetDimCount: Integer;
  public
    Function    AddDimension(DimId: LongInt; DimType : TDimensionTypes; DimOrder: Integer): Integer;
    Function    AddDimensionWithElement(DimId: LongInt; DimType : TDimensionTypes; DimOrder: Integer; strElement: String): Integer;
    Function    DeleteDimension(Index: Integer): Integer;
    Function    DimensionByOrder(DimOrder: Integer) : TMikQueryDimension;
    property    Dimension[i: Integer] : TMikQueryDimension read GetDimension;
    Property    NewValue: Double read fValue write fValue;
    property    DimensionCount: Integer read GetDimCount;
    property    DimMultiElement: Integer read fMultiElement;
    property    Legende : String read fLegende write fLegende;
    property    CubeId : LongInt read fCubeId write fCubeId;
  end;

  TMikQuerySerien = Class(TList)
  private     { Private-Deklarationen }
    fSerie : TMikQuerySerie;
  protected   { Protected-Deklarationen }
    constructor Create;
    destructor  Destroy; override;
    Function    GetSerie(Index: Integer): TMikQuerySerie;
  public
    Function    AddSerie(CubeID : LongInt; strLegende: String): Integer;
    Function    DeleteSerie(Index: Integer): Integer;
    Property    Serie[Index: Integer]: TMikQuerySerie read GetSerie;
    Property    Count;
  end;

  TMikParameter = class(TComponent)
  private
    { Private-Deklarationen }
  protected
    { Protected-Deklarationen }
  public
    { Public-Deklarationen }
    QuerySerien  : TMikQuerySerien;
    Constructor Create(AOwner : TComponent); override;
    Destructor  Destroy; override;
    procedure   ClearAllParameters;
  published
    { Published-Deklarationen }
  end;

procedure Register;

implementation

{------------------------------------------------------------------------------}
{Methoden fr TParameter                                                       }
{------------------------------------------------------------------------------}
Constructor TMikParameter.Create(AOwner : TComponent);
begin
  inherited create(AOwner);
  QuerySerien  := TMikQuerySerien.Create;
end;

{------------------------------------------------------------------------------}
Destructor  TMikParameter.Destroy;
begin
  if QuerySerien <> NIL then QuerySerien.Free;
  inherited Destroy;
end;

{------------------------------------------------------------------------------}
procedure   TMikParameter.ClearAllParameters;
begin
  if QuerySerien <> NIL then QuerySerien.Free;
  QuerySerien  := TMikQuerySerien.Create;
end;

{------------------------------------------------------------------------------}
{ Funktion fr TMikQuerySerien                                                 }
{------------------------------------------------------------------------------}
constructor TMikQuerySerien.Create;
begin
  inherited create;
end;

{------------------------------------------------------------------------------}
destructor  TMikQuerySerien.Destroy;
begin
  while count > 0 do DeleteSerie(Count-1);
  inherited destroy;
end;

{------------------------------------------------------------------------------}
Function  TMikQuerySerien.AddSerie(CubeID : LongInt; strLegende: String): Integer;
begin
  result := -1;
  fSerie := TMikQuerySerie.Create(self);
  fSerie.fCubeId := CubeId;
  fSerie.fOwner := self;
  fSerie.Legende := strLegende;
Try
  inherited Add(fSerie);
  result := count - 1;
except
end;
end;

{------------------------------------------------------------------------------}
Function  TMikQuerySerien.DeleteSerie(Index: Integer): Integer;
begin
  result := -1;
  if (Index > -1) and (Index < count) then
  begin
    fSerie := GetSerie(Index);
    if fSerie <> NIL then fSerie.Free;
    Delete(Index);
    result := 0;
  end;
end;

{------------------------------------------------------------------------------}
Function  TMikQuerySerien.GetSerie(Index: Integer): TMikQuerySerie;
begin
  result := NIL;
  if (Index > -1) and (Index < count) then
  begin
    fSerie := GetSerie(Index);
    if fSerie <> NIL then result := fSerie;
  end;
end;

{------------------------------------------------------------------------------}
{ Funktion fr TMikQuerySerie                                                  }
{------------------------------------------------------------------------------}
Constructor TMikQuerySerie.Create(AOwner: TList);
begin
  inherited create;
  fOwner := AOwner;
  Dimensions := TMikQueryDimensions.Create(self);
  NewValue := 0.0;
  fMultiElement := -1;
end;

{------------------------------------------------------------------------------}
destructor TMikQuerySerie.Destroy;
begin
  Dimensions.Free;
  inherited Destroy;
end;

{------------------------------------------------------------------------------}
Function TMikQuerySerie.AddDimension(DimId: LongInt; DimType : TDimensionTypes; DimOrder : Integer): Integer;
begin
  result := Dimensions.AddDimension(DimId,DimType,DimOrder);
end;

{------------------------------------------------------------------------------}
Function TMikQuerySerie.AddDimensionWithElement(DimId: LongInt; DimType : TDimensionTypes; DimOrder: Integer; strElement: String): Integer;
begin
  result := -1;
  Try
  result := Dimensions.AddDimension(DimId,DimType,DimOrder);
  Dimension[Result].AddElement(strElement);
  except
  end;
end;

{------------------------------------------------------------------------------}
Function TMikQuerySerie.DeleteDimension(Index: Integer): Integer;
begin
  result := Dimensions.DeleteDimension(Index);
end;

{------------------------------------------------------------------------------}
Function TMikQuerySerie.GetDimension(Index : Integer): TMikQueryDimension;
begin
  Result := Dimensions.GetDimension(Index);
end;

{------------------------------------------------------------------------------}
Function TMikQuerySerie.DimensionByOrder(DimOrder : Integer): TMikQueryDimension;
begin
  Result := Dimensions.GetDimensionByOrder(DimOrder);
end;

{------------------------------------------------------------------------------}
Function TMikQuerySerie.GetDimCount: Integer;
begin
  result := Dimensions.Count;
end;

{------------------------------------------------------------------------------}
{ Funktion fr TMikQueryDimensions                                             }
{------------------------------------------------------------------------------}
constructor TMikQueryDimensions.Create(AOwner: TMikQuerySerie);
begin
  inherited create;
  fOwner := AOwner;
  fAktDimension := -1;
end;

{------------------------------------------------------------------------------}
destructor  TMikQueryDimensions.Destroy;
begin
  while count > 0 do DeleteDimension(Count-1);
  inherited Destroy;
end;

{------------------------------------------------------------------------------}
Function TMikQueryDimensions.AddDimension(DimId: LongInt; DimType : TDimensionTypes; DimOrder: Integer): Integer;
begin
  result := -1;
  fDimension := TMikQueryDimension.Create(self);
  fDimension.fDimId := DimId;
  fDimension.fDimType := DimType;
  fDimension.fDimOrder := DimOrder;
try
  inherited Add(fDimension);
  result := count - 1;
except
end;
end;

{------------------------------------------------------------------------------}
Function    TMikQueryDimensions.DeleteDimension(Index: Integer): Integer;
begin
  result := -1;
  if (Index > -1) and (Index < count) then
  begin
    fDimension := GetDimension(Index);
    if fDimension <> NIL then fDimension.Free;
    Delete(Index);
    result := 0;
  end;
end;

{------------------------------------------------------------------------------}
Function TMikQueryDimensions.GetDimension(Index : Integer): TMikQueryDimension;
begin
  result := NIL;
  if (Index > -1) and (Index < count) then
  begin
    fDimension := GetDimension(Index);
    if fDimension <> NIL then result := fDimension;
    if Result <> NIL then fAktDimension := Index
    else                  fAktDimension := -1;
  end;
end;

{------------------------------------------------------------------------------}
Function TMikQueryDimensions.GetDimensionByOrder(DimOrder : Integer): TMikQueryDimension;
var i : Integer;
begin
  Result := NIL;
  i := 0;
  while (i < count) and (result = NIL) do
  begin
    fDimension := GetDimension(i);
    if fDimension <> NIL then
       if fDimension.fDimOrder = DimOrder then
       begin
          result := fDimension;
          fAktDimension := i;
       end;
    inc(i);
  end;
end;

{------------------------------------------------------------------------------}
{ Funktion fr TMikQueryDimension                                              }
{------------------------------------------------------------------------------}
constructor TMikQueryDimension.Create(AOwner: TMikQueryDimensions);
begin
  inherited create;
  fOwner := AOwner;
  Elements := TMikQueryElements.create(self);
end;

{------------------------------------------------------------------------------}
destructor TMikQueryDimension.Destroy;
begin
  Elements.Free;
  inherited Destroy;
end;

{------------------------------------------------------------------------------}
Function TMikQueryDimension.AddElement(Value: String): Integer;
begin
  result := Elements.AddElement(Value);
  if (Result > 0) and (fOwner.fOwner.fMultiElement = -1) then
     fOwner.fOwner.fMultiElement := fOwner.fAktDimension;
end;

{------------------------------------------------------------------------------}
Function TMikQueryDimension.DeleteElement(Index: Integer): Integer;
begin
  result := Elements.DeleteElement(Index);
end;

{------------------------------------------------------------------------------}
Function TMikQueryDimension.GetElement(Index: Integer): String;
begin
  result := Elements.GetElement(Index);
end;

{------------------------------------------------------------------------------}
Function TMikQueryDimension.GetElementCount: Integer;
begin
  result := Elements.Count;
end;

{------------------------------------------------------------------------------}
{ Funktion fr TMikQueryElements                                               }
{------------------------------------------------------------------------------}
constructor TMikQueryElements.Create(AOwner: TMikQueryDimension);
begin
  inherited create;
  fOwner := AOwner;
end;

{------------------------------------------------------------------------------}
destructor  TMikQueryElements.Destroy;
begin
  while count > 0 do DeleteElement(Count-1);
  inherited Destroy;
end;

{------------------------------------------------------------------------------}
Function TMikQueryElements.AddElement(Value: String): Integer;
begin
  result := -1;
  new(fElement);
  fElement^ := Value;
try
  inherited Add(fElement);
  result := count-1;
except
end;
end;

{------------------------------------------------------------------------------}
Function TMikQueryElements.DeleteElement(Index: Integer): Integer;
begin
  result := -1;
  if (Index > -1) and (Index < count) then
  begin
    fElement := inherited Items[Index];
    Dispose(fElement);
    Delete(Index);
    result := 0;
  end;
end;

{------------------------------------------------------------------------------}
Function TMikQueryElements.GetElement(Index: Integer): String;
begin
  result := '';
  if (Index > -1) and (Index < count) then
  begin
    fElement := inherited Items[Index];
    result := fElement^;
  end;
end;

procedure Register;
begin
  RegisterComponents('MikParameter', [TMikParameter]);
end;

initialization
{$I TMikParameter.lrs}

end.
