unit MikOlap;

interface

uses
  Windows, SysUtils, Classes, Forms, Dialogs, Controls,
  MikApi04d, MikTypes, MikDataLst, MikParameter, MikArray, LResources;

{ Forms, Graphics, Controls, Dialogs, Dialogs, Messages, }
const
    crOlapQuery1 = 25;
    crOlapQuery2 = 26;
    crOlapQuery3 = 27;
    Version      = 'TMikOLAP 1.0';

type
  TMikOLAP     = class;
  TOlapDB     = class;
  TElements   = class;
  TDimensions = class;
  TCubes      = class;

  TOlapDataEvent = procedure(Sender: TObject; iPosition: Integer; iSerie: Integer; iElement: Integer) of object;

  TData = Record
            level  : LongInt;
            Value  : Double;
            Text   : Array[0..81] of Char;
            DimId  : LongInt;
            DimText: String;
          end;

{------------------------------------------------------------------------------}
{ Typen fr Olap-Datenbank                                                     }
{------------------------------------------------------------------------------}

  TInformation = Record
                   Id    : Longint;
                   Name  : String;
                   {typ   : LongInt;}
                 end;

  TCubeInfo    = Record
                   Id    : Longint;
                   Name  : String;
                   typ   : TCubeTypes;
                 end;

  TDimensionInfo = Record
                   Id      : Longint;
                   Name    : String;
                   DimOrder: Integer;
                   typ     : TDimensionTypes;
                 end;

  TElementInfo = Record
                   Id   : LongInt;
                   Name : String;
                   level: LongInt;
                   typ  : Integer;
                   attr : TStringList;
                 end;
  {pElementInfo = ^TElementInfo;}

  TElements = class(TList)
  private    { Private-Deklarationen }
    fInfo : ^TElementInfo;
  protected  { Protected-Deklarationen }
    function  GetElementInfo(i: Integer): TElementInfo;
    Procedure SetElementInfo(i: Integer; value : TElementInfo);
    function  Add(Value: TElementInfo): Integer;
  public     { Public-Deklarationen }
    destructor  Destroy; override;
    Function GetId(Name: String): LongInt;
    Function GetName(Id : LongInt): String;
    property Items[i: Integer] : TElementInfo read GetElementInfo;
    property count;
  end;

  TDimensions = class(TList)
  private    { Private-Deklarationen }
    fInfo     : ^TDimensionInfo;
  protected  { Protected-Deklarationen }
    function  GetInformation(i: Integer): TDimensionInfo;
    function  Add(Value: TDimensionInfo): Integer;
    Constructor Create;
  public     { Public-Deklarationen }
    destructor  Destroy; override;
    Function  GetId(Name: String): LongInt;
    Function  GetName(Id : LongInt): String;
    Function  GetOrder(Id : LongInt): Integer;
    Function  GetType(Id : LongInt): TDimensionTypes;
    property  Items[i: Integer] : TDimensionInfo read GetInformation;
    property  count;
  end;

  TCubes = class(TList)
  private    { Private-Deklarationen }
    fInfo     : ^TCubeInfo;
  protected  { Protected-Deklarationen }
    function  GetInformation(i: Integer): TCubeInfo;
    function  Add(Value: TCubeInfo): Integer;
  public     { Public-Deklarationen }
    destructor  Destroy; override;
    Function  GetId(Name: String): LongInt;
    Function  GetName(Id : LongInt): String;
    property  Items[i: Integer] : TCubeInfo read GetInformation;
    property  count;
  end;

  TOlapDB = class(TList)
  private    { Private-Deklarationen }
    fInfo      : ^TInformation;
  protected  { Protected-Deklarationen }
    function    GetInformation(i: Integer): TInformation;
    function    Add(Value: TInformation): Integer;
  public     { Public-Deklarationen }
    destructor  Destroy; override;
    Function    GetId(Name: String): LongInt;
    Function    GetName(Id : LongInt): String;
    property    Items[i: Integer] : TInformation read GetInformation;
    property    count;
  end;

  TMikOLAP = class(TComponent)
  private
    { Private-Deklarationen }
        fDllFile   : TFileName;
        fActivUser : LongInt;
        fActivDB   : LongInt;
        fActivCube : LongInt;
        fActivElement : LongInt;
        fLanguage  : LongInt;
        fMultiLogin: Boolean;
        fAttrCubeIndex : Integer;
        fAttrCubeId    : LongInt;
        fUseProtokoll  : Boolean;
        fQueryCursor   : TOlapCursor;
        fOlapStatus    : TOlapStatus;
        fApiMessage    : Boolean;
        fOnGetData     : TOlapDataEvent;
        fOnSetData     : TOlapDataEvent;
        Procedure SetQueryCursor(Value: TOlapCursor);
  protected
    { Protected-Deklarationen }
        Procedure   SetDll(Filename: TFileName);
        Function    ApiGetDatabase: LongInt;
        Function    ApiGetRequiredLoginParams (Databaseid : LongInt): LongInt;
        Function    ApiOpenDB(DatabaseId: LongInt; User, Password: String; Language: LongInt): LongInt;
        Function    ApiCloseDb(User: LongInt; dbId : LongInt): LongInt;
        Function    ApiGetCubes(User: LongInt; DataBaseId: LongInt): LongInt;
        Function    ApiGetDimensions(User: LongInt; dbId: LongInt; CubeId: LongInt) : LongInt;
        Function    ApiGetElements(User: LongInt; dbId: LongInt; CubeId: LongInt; DimensionId: LongInt; AttrIncluded: Boolean): LongInt;
        Function    ApiGetAttribut(User: LongInt; dbId: LongInt; dimid: LongInt; elementname: String; var res: TStringList): LongInt;
        Function    ApiGetDataItem(User: LongInt; cubeid: LongInt; options: LongInt; p : TMikParameter; var res: TMikDataLst): LongInt;
        Function    ApiPutDataItem(User: LongInt; cubeid: LongInt; options: LongInt; p : TMikParameter): LongInt;
        Procedure   WriteProtokoll(s: String);
  public
    { Public-Deklarationen }
        OlapDB    : TOlapDB;
        Cubes     : TCubes;
        Dimensions: TDimensions;
        Elements  : TElements;
        AttrDims  : TDimensions;
        constructor Create(AOwner: TComponent); override;
        destructor  Destroy; override;
        Function    OpenDB(id: LongInt; UserName, Passwort: String): Word;
        Function    CloseDB: LongInt;
        Function    OpenCube(id: LongInt): Word;
        Function    OpenDimension(id : LongInt; AttrIncluded: Boolean): Word;
        Function    GetDatabases: LongInt;
        Function    GetAttribut(DimId : LongInt; strElement: String; var res : TStringList): Word;
        Function    GetOlapData(p : TMikParameter; var res: TMikDataLst): LongInt;
        Function    SetOlapData(p : TMikParameter): LongInt;
        Procedure   SetOlapStatus(iStatus : LongInt);
        property    ActivDbId : LongInt read fActivDB;
        property    OlapStatus: TOlapStatus read fOlapStatus;
  published
    { Published-Deklarationen }
        property DllFile: TFileName  read fDllFile write SetDll;
        property Language : LongInt read fLanguage write fLanguage;
        property MultiLogin : Boolean read fMultiLogin write fMultiLogin;
        property UseProtokoll : Boolean read fUseProtokoll write fUseProtokoll;
        property QueryCursor : TOlapCursor read fQueryCursor write SetQueryCursor;
        property RaiseApiErrorMessage: Boolean read fApiMessage write fApiMessage;
        property OnGetData       : TOlapDataEvent read fOnGetData write fOnGetData;
        property OnSetData       : TOlapDataEvent read fOnSetData write fOnSetData;
  end;


procedure Register;

implementation

{$R olapcursor.res}


CONST
    moFirst   : LongInt = 1;
    moNext    : LongInt = 0;
var
    iOlapCursor : TCursor;
    fProto      : TextFile;

{------------------------------------------------------------------------------}
{ Funktionen fr TMikOLAP                                                       }
{------------------------------------------------------------------------------}
Procedure TMikOLAP.WriteProtokoll(s: String);
begin
try
  reset(fProto);
  Writeln(fProto,s);
  closefile(fProto);
except
end;
end;

constructor TMikOLAP.Create(AOwner: TComponent);
begin
  inherited create(AOwner);
  if fUseProtokoll = True then AssignFile(fProto,ExtractFilePath(paramstr(0))+'MikApi.pro');
  fActivUser := 0;
  fActivDB   := 0;
  MultiLogin := true;
  Language := 0;
  QueryCursor := tocrCusor1;
  OlapDB     := TOlapDB.Create;
  Cubes      := TCubes.Create;
  Dimensions := TDimensions.Create;
  AttrDims   := TDimensions.Create;
  Elements   := TElements.Create;
  Screen.Cursors[crOlapQuery1] := LoadCursor(HInstance, 'QUERY1');
  Screen.Cursors[crOlapQuery2] := LoadCursor(HInstance, 'QUERY2');
  Screen.Cursors[crOlapQuery3] := LoadCursor(HInstance, 'QUERY3');
  if fUseProtokoll = true then WriteProtokoll('TMikOLAP created');
end;

{------------------------------------------------------------------------------}
Procedure TMikOLAP.SetOlapStatus(iStatus : LongInt);
begin
  case iStatus of
   0  : fOlapStatus := moOkay;
   1  : fOlapStatus := moNoMoreData;
   3  : fOlapStatus := moServerNotAvaliable;
   -1 : fOlapStatus := moServerError;
   -2 : fOlapStatus := moNoData;
   -3 : fOlapStatus := moServerBusy;
   -4 : fOlapStatus := moServerBlocked1;
   -5 : fOlapStatus := moServerBlocked2;
   -6 : fOlapStatus := moServerUpdated;
   -7 : fOlapStatus := moUserAlreadyLogged;
  end;
  if fApiMessage and (iStatus <> 0) then
  begin
    case iStatus of
      3 : MessageDlg('Server nicht verfgbar',mtWarning,[mbOk],0);
     -1 : MessageDlg('Serverfehler',mtWarning,[mbOk],0);
     -2 : MessageDlg('keine Daten',mtWarning,[mbOk],0);
     -3 : MessageDlg('Server ist beschftigt, kein Login mglich',mtWarning,[mbOk],0);
     -4 : MessageDlg('Server ist blockiert, kein Login mglich',mtWarning,[mbOk],0);
     -5 : MessageDlg('Server ist blockiert, kein Login mglich',mtWarning,[mbOk],0);
     -6 : MessageDlg('Server wurde aktualisiert',mtWarning,[mbOk],0);
     -7 : MessageDlg('Anwender bereits eingelogged',mtWarning,[mbOk],0);
    end;
  end;
end;

{------------------------------------------------------------------------------}
Procedure TMikOLAP.SetQueryCursor(Value: TOlapCursor);
begin
  if Value <> fQueryCursor then
  begin
    fQueryCursor := Value;
    Case Value of
      tocrNone   : iOlapCursor := -1;
      tocrCusor1 : iOlapCursor := crOlapQuery1;
      tocrCusor2 : iOlapCursor := crOlapQuery2;
      tocrCusor3 : iOlapCursor := crOlapQuery3;
    end;
  end;
end;

{------------------------------------------------------------------------------}
Procedure TMikOLAP.SetDll(Filename: TFileName);
begin
  if Filename = '' then fDLLFile := 'C:\MIKOLAP\MIKAPIO.DLL'
  else                  fDLLFile := Filename;
  MikOlapPath := ExtractFilePath(fDLLFile);
  MikOlapAPI := copy(fDLLFile,length(MikOlapPath)+1,length(fDLLFile)-length(MikOlapPath));
end;

{------------------------------------------------------------------------------}
destructor  TMikOLAP.Destroy;
begin
  if fActivDB <> 0 then ApiCloseDB(fActivUser,fActivDB);
  if OlapDB     <> nil then OlapDB.Free;
  if Cubes      <> nil then Cubes.Free;
  if Dimensions <> nil then Dimensions.Free;
  if Elements   <> nil then Elements.Free;
  if AttrDims   <> nil then AttrDims.Free;
  inherited Destroy;
end;

{------------------------------------------------------------------------------}
Function   TMikOLAP.GetDatabases: LongInt;
var OldCursor : TCursor;
begin
  if fUseProtokoll = true then WriteProtokoll('TMikOLAP.GetDatabases');
  OldCursor := Screen.Cursor;
  if iOlapCursor <> -1 then Screen.Cursor := iOlapCursor;
  result := ApiGetDataBase;
  Screen.Cursor := OldCursor;
end;

{------------------------------------------------------------------------------}
Function    TMikOLAP.OpenDB(id: LongInt; UserName, Passwort: String): Word;
var OldCursor : TCursor;
begin
  if fUseProtokoll = true then WriteProtokoll('TMikOLAP.OpenDB '+IntToStr(id)+' '+Name+' '+Passwort);
  OldCursor := Screen.Cursor;
  if iOlapCursor <> -1 then Screen.Cursor := iOlapCursor;
  result := 0;
  if Id <> 0 then
  begin
    fActivUser := ApiOpenDB(Id,Name,Passwort,fLanguage);
    if fActivUser = 0 then
    begin
      Result := 0;
      fActivDB := 0;
    end
    else
    begin
      fActivDB := Id;
      Result := ApiGetCubes(fActivUser,fActivDB);
    end;
  end;
  Screen.Cursor := OldCursor;
end;

{------------------------------------------------------------------------------}
Function TMikOLAP.ApiOpenDB(DatabaseId: LongInt; User, Password: String; Language: LongInt): LongInt;
var iLoginMode : LongInt;
begin
  if fActivDB > 0 then
  begin
    ApiCloseDb(fActivUser,fActivDB);
    fActivUser := 0;
    fActivDB   := 0;
  end;
  if fMultiLogin = True then
     iLoginMode := 88
  else
     iLoginMode := 0;
  result := moLogin (0,iLoginMode,DatabaseId,'',PCHAR(User),PCHAR(Password),Language);
  SetOlapStatus(result);
  if result <> 0 then
  begin
    fActivUser := result;
    fActivDB   := DatabaseId;
  end;
end;

{------------------------------------------------------------------------------}
Function    TMikOLAP.CloseDB: LongInt;
var OldCursor : TCursor;
begin
  if fUseProtokoll = true then WriteProtokoll('TMikOLAP.CloseDB');
  OldCursor := Screen.Cursor;
  if iOlapCursor <> -1 then Screen.Cursor := iOlapCursor;
  Application.ProcessMessages;
  result := 0;
  if (fActivUser > 0) and (fActivDB > 0) then
     result := ApiCloseDb(fActivUser,fActivDB);
  Screen.Cursor := OldCursor;
end;

{------------------------------------------------------------------------------}
Function    TMikOLAP.OpenCube(id: LongInt): Word;
var OldCursor : TCursor;
begin
  if fUseProtokoll = true then WriteProtokoll('TMikOLAP.OpenCube '+IntToStr(id));
  OldCursor := Screen.Cursor;
  if iOlapCursor <> -1 then Screen.Cursor := iOlapCursor;
  Application.ProcessMessages;
  result := 0;
  fActivCube := 0;
  Try
    if (fActivDB <> 0) and (Id <> 0) then
    begin
       result := ApiGetDimensions(fActivUser,fActivDB,Id);
       fActivCube := id;
    end;
  except
  end;
  Screen.Cursor := OldCursor;
end;

{------------------------------------------------------------------------------}
Function    TMikOLAP.OpenDimension(id : LongInt; AttrIncluded: Boolean): Word;
var OldCursor : TCursor;
begin
  if fUseProtokoll = true then WriteProtokoll('TMikOLAP.OpenDimension '+IntToStr(id));
  OldCursor := Screen.Cursor;
  if iOlapCursor <> -1 then Screen.Cursor := iOlapCursor;
  Application.ProcessMessages;
  result := 0;
  fActivElement := 0;
  Try
    if (fActivUser > 0) and (fActivDB > 0) and (fActivCube > 0) and (id > 0) then
    begin
       result := ApiGetElements(fActivUser, fActivDB, fActivCube, Id, AttrIncluded);
       fActivElement := id;
    end;
  except
  end;
  Screen.Cursor := OldCursor;
end;

{------------------------------------------------------------------------------}
Function  TMikOLAP.GetAttribut(DimId : LongInt; strElement: String; var res : TStringList): Word;
begin
  if fUseProtokoll = true then WriteProtokoll('TMikOLAP.GetAttrbibute '+IntToStr(Dimid));
  result := 0;
  try
    if (fActivUser > 0) and (fActivDB > 0) and (fActivCube > 0) and (Dimid > 0) and (strElement <> '') then
       result := ApiGetAttribut(fActivUser,fActivDB,DimId,strElement,res)
  except
  end;
end;

{------------------------------------------------------------------------------}
Function  TMikOLAP.GetOlapData(p : TMikParameter; var res: TMikDataLst): LongInt;
var OldCursor : TCursor;
begin
  if fUseProtokoll = true then WriteProtokoll('TMikOLAP.GetOlapData start');
  OldCursor := Screen.Cursor;
  if iOlapCursor <> -1 then Screen.Cursor := iOlapCursor;
  Application.ProcessMessages;
  Result := 0;
  if (fActivUser > 0) and (fActivDB > 0) then
     result := ApiGetDataItem(fActivUser,fActivCube,0,p,res);
  Screen.Cursor := OldCursor;
  if fUseProtokoll = true then WriteProtokoll('TMikOLAP.GetOlapData end');
end;

{------------------------------------------------------------------------------}
Function  TMikOLAP.SetOlapData(p : TMikParameter): LongInt;
var OldCursor : TCursor;
begin
  if fUseProtokoll = true then WriteProtokoll('TMikOLAP.SetOlapData start');
  OldCursor := Screen.Cursor;
  if iOlapCursor <> -1 then Screen.Cursor := iOlapCursor;
  Application.ProcessMessages;
  Result := 0;
  if (fActivUser > 0) and (fActivDB > 0) then
     result := ApiPutDataItem(fActivUser,fActivCube,0,p);
  Screen.Cursor := OldCursor;
  if fUseProtokoll = true then WriteProtokoll('TMikOLAP.SetOlapData end');
end;

{------------------------------------------------------------------------------}
Function TMikOLAP.ApiGetDatabase: LongInt;
var dbId : LongInt;
    dbName: Array[0..255] of Char;
    res : LongInt;
    ti : TInformation;
begin
    OlapDB.Clear;
    res := moGetDatabase(0,@dbId,@dbName,moFirst);
    while res = 0 do
    begin
      Application.ProcessMessages;
      ti.Id := dbId;
      ti.Name := UpperCase(dbname);
      OlapDB.Add(ti);
      res := moGetDatabase(0,@dbId,@dbName,moNext);
    end;
    SetOlapStatus(res);
    Result := OlapDB.count;
end;

{------------------------------------------------------------------------------}
Function TMikOLAP.ApiGetRequiredLoginParams (Databaseid : LongInt): LongInt;
begin
  result := moGetRequiredLoginParams (Databaseid);
  SetOlapStatus(result);
end;

{------------------------------------------------------------------------------}
Function TMikOLAP.ApiCloseDb(User: LongInt; dbId : LongInt): LongInt;
begin
  result := -7;
  if User > 0 then
  begin
    result := moLogout(User,dbId);
    SetOlapStatus(result);
    if result <> 0 then
    begin
      fActivUser := 0;
      fActivDB   := 0;
    end;
  end;
end;

{------------------------------------------------------------------------------}
Function TMikOLAP.ApiGetCubes(User: LongInt; DataBaseId: LongInt): LongInt;
var res : LongInt;
    CubeID   : LongInt;
    CubeType : LongInt;
    CubeName : Array[0..255] of Char;
    ci       : TCubeInfo;
begin
  res := 0;
  cubes.Clear;
  Try
    res := moGetCube(User,DataBaseId,@CubeId,@CubeType,@CubeName,moFirst);
    while res = 0 do
    begin
      Application.ProcessMessages;
      ci.Id   := CubeId;
      ci.Name := StrPas(CubeName);
      ci.typ  := TCubeTypes(CubeType);
      cubes.Add(ci);
      if ci.typ = moCiAttribute then
      begin
        fAttrCubeIndex := cubes.count-1;
        fAttrCubeId    := ci.Id;
      end;
      res := moGetCube(User,DataBaseId,@CubeId,@CubeType,@CubeName,moNext);
    end;
  except
  end;
  SetOlapStatus(res);
  result := cubes.count;
end;

{------------------------------------------------------------------------------}
Function  TMikOLAP.ApiGetDimensions(User: LongInt; dbId: LongInt; CubeId: LongInt) : LongInt;
var DimID   : LongInt;
    DimType : LongInt;
    DimName : Array[0..255] of Char;
    res     : LongInt;
    ti      : TDimensionInfo;
    iOrder  : Integer;
begin
  result := 0;
  Dimensions.clear;
  iOrder := 0;
  res := 0;
Try
  if (User > 0) and (dbId > -1) and (CubeId > -1) then
  begin
    res := moGetDimension(User,dbId,CubeId,@DimId,@DimType,@DimName,moFirst);
    while res = 0 do
    begin
      Application.ProcessMessages;
      ti.Id       := DimId;
      ti.typ      := TDimensionTypes(DimType);
      ti.Name     := strPas(DimName);
      ti.DimOrder := iOrder;
      inc(iOrder);
      if CubeId = fAttrCubeId then AttrDims.Add(ti);
      Dimensions.Add(ti);
      res := moGetDimension(User,dbId,CubeId,@DimId,@DimType,@DimName,moNext);
    end;
  end;
  SetOlapStatus(res);
  result := Dimensions.count;
except
end; {Try}
end;

{------------------------------------------------------------------------------}
Function  TMikOLAP.ApiGetElements(User: LongInt; dbId: LongInt; CubeId: LongInt; DimensionId: LongInt; AttrIncluded: Boolean): LongInt;
var pCondi, pResult : _Params;
    res : LongInt;
    te  : TElementInfo;
    s   : sc;
    iLastLevel   : Integer;
begin
  Elements.Clear;
  FillChar(pCondi,SizeOf(pCondi),0);
  moSetElementInfo(User,dbId,CubeID,DimensionId,0,@pCondi);

  pResult.cUsage := '1';
  pResult.lpszTxt := @s;
  res := moGetElement(User,@pResult);
  iLastLevel := pResult.level;
  while res = 0 do
  begin
    Application.ProcessMessages;
    te.level := pResult.level;
    te.Name  := StrPas(s);
    te.Typ   := 0;
    if AttrIncluded then
    begin
      te.attr := TStringList.Create;
      ApiGetAttribut(User,dbId,DimensionId,te.Name,te.attr);
    end;
    Elements.Add(te);
    if pResult.level > iLastlevel then iLastLevel := pResult.level;
    res := moGetElement(User,@pResult);
    pResult.cUsage := '0';
  end;
  SetOlapStatus(res);
  result := Elements.count;
end;

{------------------------------------------------------------------------------}
Function  TMikOLAP.ApiGetAttribut(User: LongInt; dbId: LongInt; dimid: LongInt; elementname: String; var res: TStringList): LongInt;
var msg : LongInt;
    resText : Array[0..255] of Char;
    cElementName : Array[0..255] of Char;
    i : Integer;
begin
  msg := 0;
  if res = nil then res := TStringList.Create;
  For i := 0 to AttrDims.count-1 do
  begin
    Application.ProcessMessages;
    fillchar(resText,SizeOf(resText),0);
    fillchar(cElementName,SizeOf(cElementName),0);
    strPcopy(cElementName,ElementName);
    msg := moGetAttribute(User, dbId, dimid, @cElementName, AttrDims.Items[i].Id, @resText, moFirst);
    while msg = 0 do
    begin
      Application.ProcessMessages;
      res.Add(strpas(resText));
      fillchar(resText,SizeOf(resText),0);
      msg := moGetAttribute(User, dbId, dimid, @cElementName, AttrDims.Items[i].Id, @resText, moNext);
    end;
  end;
  SetOlapStatus(msg);
  result := res.Count;
end;

{------------------------------------------------------------------------------}
Function TMikOLAP.ApiGetDataItem(User: LongInt; cubeid: LongInt; options: LongInt; p : TMikParameter; var res: TMikDataLst): LongInt;
var di : TMikArray;
    iParam, iPos, iRes, iSerien, iUnique, iReihen : Integer;
    sParam : String;
    ic : LongInt;
    resInfo : Params;
    td : TData;
    sText : String;
    msg : LongInt;
begin
  ic := 0;
  result := 0;
  msg := 0;
  di := TMikArray.Create;
  FillChar(resInfo,Sizeof(resinfo),0);
  FillChar(td,Sizeof(td),0);
  FillChar(di.arr,SizeOf(di.arr),#0);
  resInfo.lpszTxt := @td.Text;
  iPos := -1;
  For iSerien := 0 to p.QuerySerien.Count-1 do
  begin
    Application.ProcessMessages;
    iRes := res.DataSeries.AddSerie;
    res.DataSeries.Serie[iRes].SeriesInfo.SerieTyp := tstOlapData;
    res.DataSeries.Serie[iRes].SeriesInfo.Legend := p.QuerySerien.Serie[iSerien].Legende;
    For iUnique := 0 to p.QuerySerien.Serie[iSerien].DimensionCount-1 do
    begin
      if (p.QuerySerien.Serie[iSerien].Dimension[iUnique].ElementCount > 1) and (iPos = -1) then
         iPos := p.QuerySerien.Serie[iSerien].Dimension[iUnique].DimensionOrder
      else
         di.SetValue(p.QuerySerien.Serie[iSerien].Dimension[iUnique].DimensionOrder,
                     p.QuerySerien.Serie[iSerien].Dimension[iUnique].Element[0])
    end;
    di.Arr[0].subsettype := 1;
    For iReihen := 0 to p.QuerySerien.Serie[iSerien].Dimension[p.QuerySerien.Serie[iSerien].DimMultiElement].ElementCount-1 do
    begin
      inc(ic);
      Application.ProcessMessages;
      di.SetValue(iPos,p.QuerySerien.Serie[iSerien].Dimension[p.QuerySerien.Serie[iSerien].DimMultiElement].Element[iReihen]);
      if fUseProtokoll = true then
      begin
        For iParam := 0 to di.Count do sParam := sParam+di.GetValue(iParam);
{        if fUseProtokoll = true WriteProtokoll('TMikOLAP.GetOlapData ');}
      end;
      if Assigned(fOnGetData) then fOnGetData(self,ic,iSerien,iReihen);
      msg := moGetDataItems(User,cubeid,options,@di.arr,@resInfo);
      if StrPas(resInfo.lpszTxt) = '' then
         sText := p.QuerySerien.Serie[iSerien].Dimension[p.QuerySerien.Serie[iSerien].DimMultiElement].Element[iReihen]
      else
         sText := StrPas(resInfo.lpszTxt);

      res.DataSeries.Serie[iRes].AddData(resInfo.level,resInfo.dValue,sText);
    end;
    di.InitArray;
  end;
  SetOlapStatus(msg);
  di.free;
end;


{------------------------------------------------------------------------------}
Function  TMikOLAP.ApiPutDataItem(User: LongInt; cubeid: LongInt; options: LongInt; p: TMikParameter): LongInt;
var di : TMikArray;
    iSerien, iUnique : Integer;
    resInfo : Params;
    td : TData;
    ic, res : LongInt;
begin
  result := 0;
  res := 0;
  ic := 0;
  di := TMikArray.Create;
  FillChar(resInfo,Sizeof(resinfo),0);
  FillChar(td,Sizeof(td),0);
  FillChar(di.arr,SizeOf(di.arr),#0);
  resInfo.lpszTxt := @td.Text;
  For iSerien := 0 to p.QuerySerien.Count-1 do
  begin
    inc(ic);
    Application.ProcessMessages;
    For iUnique := 0 to p.QuerySerien.Serie[iSerien].DimensionCount-1 do
    begin
      di.SetValue(p.QuerySerien.Serie[iSerien].Dimension[iUnique].DimensionOrder,
                  p.QuerySerien.Serie[iSerien].Dimension[iUnique].Element[0])
    end;
    di.Arr[0].subsettype := 1;
    if Assigned(fOnSetData) then fOnSetData(self,ic,iSerien,0);
    resInfo.dValue :=  p.QuerySerien.Serie[iSerien].NewValue;
{    if Assigned(fOnSetData) then fOnSetData(self,ic,iSerien,iReihen);}
    res := moPutDataItems(User,cubeid,options,@di.arr,@resInfo);
    di.InitArray;
  end;
  di.free;
  SetOlapStatus(res);
end;

{------------------------------------------------------------------------------}
{ Funktion fr TOlapDB                                                         }
{------------------------------------------------------------------------------}
destructor TOlapDB.Destroy;
var p : ^TInformation;
begin
  while count > 0 do
  begin
    p := inherited Items[0];
    Dispose(p);
    Delete(0);
  end;
  inherited Destroy;
end;

{------------------------------------------------------------------------------}
function TOlapDB.GetInformation(i: Integer): TInformation;
var p : ^TInformation;
begin
  p := inherited Items[i];
  result.Id     := p^.Id;
  result.Name   := p^.Name;
  {result.Typ    := p.Typ;}
end;

{------------------------------------------------------------------------------}
function TOlapDB.Add(Value: TInformation): Integer;
begin
  new(fInfo);
  fInfo^.Id    := Value.Id;
  fInfo^.Name  := Value.Name;
  {fInfo^.Typ   := Value.Typ;}
  inherited Add(fInfo);
  result := count-1;
end;

{------------------------------------------------------------------------------}
Function TOlapDB.GetId(Name: String): LongInt;
var i : Integer;
    boFound : Boolean;
    ti : ^TElementInfo;
begin
  FillChar(ti,SizeOf(ti),0);
  result := -1;
  i := 0;
  boFound := False;
  while not boFound do
  begin
    ti := inherited Items[i];
    if ti^.Name = UpperCase(Name) then boFound := True
    else                   inc(i);
    if i = count then Exit
  end;
  if boFound then result := ti^.Id;
end;

{------------------------------------------------------------------------------}
Function TOlapDB.GetName(Id : LongInt): String;
var i : Integer;
    boFound : Boolean;
    ti : ^TInformation;
begin
  FillChar(ti,SizeOf(ti),0);
  result := '';
  i := 0;
  boFound := False;
  while not boFound do
  begin
    ti := inherited Items[i];
    if ti^.Id = Id then boFound := True
    else                   inc(i);
    if i = count then Exit
  end;
  if boFound then result := ti^.Name;
end;

{------------------------------------------------------------------------------}
{ Funktion fr TCubes                                                          }
{------------------------------------------------------------------------------}
destructor TCubes.Destroy;
begin
  while count > 0 do Delete(0);
  inherited Destroy;
end;

{------------------------------------------------------------------------------}
function TCubes.GetInformation(i: Integer): TCubeInfo;
var p : ^TCubeInfo;
begin
  if (i >= 0) and (i < Count) then
  begin
    p := inherited Items[i];
    result.Id     := p^.Id;
    result.Name   := p^.Name;
    result.typ    := p^.typ;
  end
  else
  begin
    result.Id     := 0;
    result.Name   := '';
    result.typ    := moCiNoResult;
  end;
end;

{------------------------------------------------------------------------------}
function TCubes.Add(Value: TCubeInfo): Integer;
begin
  new(fInfo);
  fInfo^.Id    := Value.Id;
  fInfo^.Name  := Value.Name;
  fInfo^.typ   := Value.typ;
  inherited Add(fInfo);
  result := count-1;
end;

{------------------------------------------------------------------------------}
Function TCubes.GetId(Name: String): LongInt;
var i : Integer;
    boFound : Boolean;
    ti : ^TCubeInfo;
begin
  FillChar(ti,SizeOf(ti),0);
  result := -1;
Try
  i := 0;
  boFound := False;
  while not boFound do
  begin
    ti := inherited Items[i];
    if ti^.Name = Name then boFound := True
    else                   inc(i);
    if i = count then Exit
  end;
  if boFound then result := ti^.Id;
except
end; {try}
end;

{------------------------------------------------------------------------------}
Function TCubes.GetName(Id : LongInt): String;
var i : Integer;
    boFound : Boolean;
    ti : ^TCubeInfo;
begin
  FillChar(ti,SizeOf(ti),0);
  result := '';
  i := 0;
  boFound := False;
  while not boFound do
  begin
    ti := inherited Items[i];
    if ti^.Id = Id then boFound := True
    else                   inc(i);
    if i = count then Exit
  end;
  if boFound then result := ti^.Name;
end;

{------------------------------------------------------------------------------}
{ Funktion fr TDimensions                                                     }
{------------------------------------------------------------------------------}
constructor TDimensions.Create;
begin
  inherited Create;
end;

{------------------------------------------------------------------------------}
destructor TDimensions.Destroy;
begin
  while count > 0 do Delete(0);
  inherited Destroy;
end;

{------------------------------------------------------------------------------}
function TDimensions.GetInformation(i: Integer): TDimensionInfo;
var p : ^TDimensionInfo;
begin
  if (i >= 0) and (i < count) then
  begin
    p := inherited Items[i];
    result.Id       := p^.Id;
    result.Name     := p^.Name;
    result.typ      := p^.typ;
    result.DimOrder := p^.DimOrder;
  end
  else
  begin
    result.Id       := 0;
    result.Name     := '';
    result.typ      := moDiNoResult;
    result.DimOrder := -1;
  end
end;

{------------------------------------------------------------------------------}
function TDimensions.Add(Value: TDimensionInfo): Integer;
begin
  new(fInfo);
  fInfo^.Id       := Value.Id;
  fInfo^.Name     := Value.Name;
  fInfo^.typ      := Value.typ;
  fInfo^.DimOrder := Value.DimOrder;
  inherited Add(fInfo);
  result := count-1;
end;

{------------------------------------------------------------------------------}
Function TDimensions.GetId(Name: String): LongInt;
var i : Integer;
    boFound : Boolean;
    ti : ^TDimensionInfo;
begin
  FillChar(ti,SizeOf(ti),0);
  result := -1;
TRY
  i := 0;
  boFound := False;
  while not boFound do
  begin
    ti := inherited Items[i];
    if ti^.Name = Name then boFound := True
    else                   inc(i);
    if i = count then Exit
  end;
  if boFound then result := ti^.Id;
except
end; {try}
end;

{------------------------------------------------------------------------------}
Function  TDimensions.GetType(Id : LongInt): TDimensionTypes;
var i : Integer;
    boFound : Boolean;
    ti : ^TDimensionInfo;
begin
  FillChar(ti,SizeOf(ti),0);
  result := modiAny;
  i := 0;
  boFound := False;
  while not boFound do
  begin
    ti := inherited Items[i];
    if ti^.id = id then boFound := True
    else               inc(i);
    if i = count then Exit
  end;
  if boFound then result := ti^.typ;
end;

{------------------------------------------------------------------------------}
Function TDimensions.GetName(Id : LongInt): String;
var i : Integer;
    boFound : Boolean;
    ti : ^TDimensionInfo;
begin
  FillChar(ti,SizeOf(ti),0);
  result := '';
  i := 0;
  boFound := False;
  while not boFound do
  begin
    ti := inherited Items[i];
    if ti^.Id = Id then boFound := True
    else                   inc(i);
    if i = count then Exit
  end;
  if boFound then result := ti^.Name;
end;

{------------------------------------------------------------------------------}
Function  TDimensions.GetOrder(Id : LongInt): Integer;
var i : Integer;
    boFound : Boolean;
    ti : ^TDimensionInfo;
begin
  FillChar(ti,SizeOf(ti),0);
  result := -1;
  i := 0;
  boFound := False;
  while not boFound do
  begin
    ti := inherited Items[i];
    if ti^.Id = Id then boFound := True
    else                   inc(i);
    if i = count then Exit
  end;
  if boFound then result := ti^.DimOrder;
end;

{------------------------------------------------------------------------------}
{ Funktion fr TElements                                                       }
{------------------------------------------------------------------------------}
destructor TElements.Destroy;
var p : ^TElementInfo;
begin
  while count > 0 do
  begin
    p := inherited Items[0];
TRY
    if p^.attr <> NIL then p^.attr.Free;
EXCEPT
END;
    Delete(0);
  end;
  inherited Destroy;
end;

{------------------------------------------------------------------------------}
function TElements.GetElementInfo(i: Integer): TElementInfo;
var p : ^TElementInfo;
begin
  p := inherited Items[i];
  result.Id     := p^.Id;
  result.Name   := p^.Name;
  result.Level  := p^.Level;
end;

{------------------------------------------------------------------------------}
Procedure TElements.SetElementInfo(i: Integer; value: TElementInfo);
var p : ^TElementInfo;
begin
  if (i > -1) and (i < count) then
  begin
    p := inherited Items[i];
    p^.Id     := value.Id;
    p^.Name   := value.Name;
    p^.Level  := value.Level;
    p^.typ    := value.typ;
  end;
end;

{------------------------------------------------------------------------------}
function TElements.Add(Value: TElementInfo): Integer;
begin
  new(fInfo);
  fInfo^.Id    := Value.Id;
  fInfo^.Name  := Value.Name;
  fInfo^.level := Value.level;
  fInfo^.attr := TStringList.Create;
  inherited Add(fInfo);
  result := count-1;
end;

{------------------------------------------------------------------------------}
Function TElements.GetId(Name: String): LongInt;
var i : Integer;
    boFound : Boolean;
    ti : ^TElementInfo;
begin
  FillChar(ti,SizeOf(ti),0);
  result := -1;
  i := 0;
  boFound := False;
  while not boFound do
  begin
    ti := inherited Items[i];
    if ti^.Name = Name then boFound := True
    else                   inc(i);
    if i = count then Exit
  end;
  if boFound then result := ti^.Id;
end;

{------------------------------------------------------------------------------}
Function TElements.GetName(Id : LongInt): String;
var i : Integer;
    boFound : Boolean;
    ti : ^TElementInfo;
begin
  FillChar(ti,SizeOf(ti),0);
  result := '';
  i := 0;
  boFound := False;
  while not boFound do
  begin
    ti := inherited Items[i];
    if ti^.Id = Id then boFound := True
    else                   inc(i);
    if i = count then Exit
  end;
  if boFound then result := ti^.Name;
end;

procedure Register;
begin
  RegisterComponents('MikOlap', [TMikOLAP]);
end;

initialization
{$I TMikOLAP.lrs}

end.
